#v1.0
#v0.9 - All research graph via menu & mouse click
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx
from tkcalendar import Calendar, DateEntry

from datetime import date
import matplotlib
from matplotlib.pyplot import Figure
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import interactive
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators
from addnewmodifyscript import classAddNewModifyScript
from testdata import *

class classAllGraphs(Toplevel):
    def __init__(self, master=None, argistestmode=False, argkey='XXXX', argscript='', 
        argmenucalled=False, arggraphid=-1, argoutputtree=None, argdatafolder='./scriptdata', **kw):
        super().__init__(master=master, **kw)
        self.wm_state(newstate='zoomed')
        self.wm_title("Graphs")

        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)

        self.key = argkey
        self.script = argscript
        self.bool_test = argistestmode
        self.bool_menucalled = argmenucalled
        if(arggraphid <= 1):
            self.graphid = arggraphid
        else:
            self.graphid = arggraphid + 1
        self.output_tree = argoutputtree
        self.datafolderpath = argdatafolder
        
        try:
            self.fromdt = date.today()
            self.fromdt = self.fromdt.replace(year=self.fromdt.year-1)
        except ValueError:
            dself.fromdt = self.fromdt.replace(year=self.fromdt.year-1, day=self.fromdt.day-1)

        self.menu= Menu(master=self)
        self.popup_ongraph = Menu(self.menu, tearoff=0)
        self.popup_ongraph.add_command(label="Selected Graph: ")        
        self.popup_ongraph.add_separator()
        self.popup_ongraph.add_command(label="Clear Graph", command=self.OnClearGraph)        
        self.popup_ongraph.add_command(label="Analysis", command=self.OnAnalysis)        

        self.event = None

        self.f = Figure(figsize=(12.63,8), dpi=100, facecolor='w', edgecolor='k', tight_layout=True, linewidth=0.5)
        self.output_canvas=FigureCanvasTkAgg(self.f, master=self)
        self.toolbar_frame=ttk.Frame(master=self, borderwidth=5, relief='sunken')
        self.toolbar = NavigationToolbar2Tk(self.output_canvas, self.toolbar_frame)
        #self.ax = [None, None, None, None, None, None, None, None, None]
        #for i in range(9):
        self.ax = self.f.add_subplot(1, 1, 1)
        self.output_canvas.mpl_connect('pick_event', self.OnPick)
        #self.ax[i] = self.f.add_axes(self.ax[i])

        self.frame1 = ttk.Frame(self, borderwidth=5, relief="sunken")
        self.search_symbol_label = ttk.Label(self.frame1, text='Search Symbol: ')
        self.search_symbol_combo_text = StringVar()
        self.search_symbol_combo = ttk.Combobox(self.frame1, width=40, 
                textvariable=self.search_symbol_combo_text, state='normal')
        self.search_symbol_combo.bind('<Return>', self.commandEnterKey)
        self.search_symbol_combo.bind('<<ComboboxSelected>>', self.OnScriptSelectionChanged)

        self.btn_search_script = ttk.Button(self.frame1, text="Search", command=self.btnSearchScript)
        self.btn_add_script = ttk.Button(self.frame1, text="Add", command=self.btnAddScript)

        self.graph_select_label = ttk.Label(self.frame1, text='Select Graph: ')
        self.graph_select_combo_text = StringVar()
        self.graph_select_combo = ttk.Combobox(self.frame1, width=30, textvariable=self.graph_select_combo_text,
                values=['Daily price', 'Intraday', 'Simple moving avg', 'Volume weighted avg price', 
                'Relative strength index', 'Avg directional movement index', 'Stochastic oscillator',
                'Moving average convergence/divergence', 'Aroon', 'Bollinger bands', 'OHCL-Candlestick'], state='readonly')
        
        self.graph_select_combo_text.set('Daily Price')
        self.graph_select_combo.current(0)
        self.graph_select_combo.bind('<<ComboboxSelected>>', self.OnGraphSelectionChanged)

        self.from_date_text = StringVar()
        self.from_date = DateEntry(master=self.frame1, width=12, year=self.fromdt.year, 
                                month=self.fromdt.month, day=self.fromdt.day, 
                                background='darkblue', foreground='white', borderwidth=2,
                                textvariable=self.from_date_text, date_pattern='y-m-d')

        self.btn_show_selected_graph = ttk.Button(self.frame1, text="Show Selected", command=self.btnShowSelectedGraph)
        self.btn_clear_selected_graph = ttk.Button(self.frame1, text="Clear Selected", command=self.btnClearSelectedGraph)
        self.btn_clear_all_graph = ttk.Button(self.frame1, text="Clear All", command=self.btnClearAllGraph)
        self.btn_cancel = ttk.Button(self.frame1, text="Close", command=self.OnClose)

        self.notebook = ttk.Notebook(self)
        self.fDailyPrice = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken", padding=5)
        self.fIntraday = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fSimpleMovingAvg = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fVolumeweightedavgprice = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fRelativeStrengthIndex = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fAvgdirectionalmovementindex = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fstochasticoscillator = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fmovingaverageconvergencedivergence = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fAroon = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fBollingerbands = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")
        self.fCandlestick = ttk.Frame(master=self.notebook, borderwidth=5, relief="sunken")

        self.notebook.add(self.fDailyPrice, text='Daily Price')
        self.notebook.add(self.fIntraday, text='Intraday')
        self.notebook.add(self.fSimpleMovingAvg, text = 'SMA')
        self.notebook.add(self.fVolumeweightedavgprice, text = 'VWAP')
        self.notebook.add(self.fRelativeStrengthIndex, text = 'RSI')
        self.notebook.add(self.fAvgdirectionalmovementindex, text='ADX')
        self.notebook.add(self.fstochasticoscillator, text='Stochastic')
        self.notebook.add(self.fmovingaverageconvergencedivergence, text='MACD')
        self.notebook.add(self.fAroon, text='AROON')
        self.notebook.add(self.fBollingerbands, text='BBANDS')
        self.notebook.add(self.fCandlestick, text='OHLC-Candlestick')

        #1 Daily
        self.outputsize_label1 = ttk.Label(self.fDailyPrice, text='Output size: ')
        self.outpusize_combo_text1 = StringVar()
        self.outputsize_combo1 = ttk.Combobox(self.fDailyPrice, width=10, textvariable=self.outpusize_combo_text1, 
                                            values=['compact', 'full'], state='readonly')
        self.outpusize_combo_text1.set('compact')
        self.outputsize_combo1.current(0)
        #2 Intraday
        self.outputsize_label2 = ttk.Label(self.fIntraday, text='Output size: ')
        self.outpusize_combo_text2 = StringVar()
        self.outputsize_combo2 = ttk.Combobox(self.fIntraday, width=10, textvariable=self.outpusize_combo_text2, 
                                            values=['compact', 'full'], state='readonly')
        self.outpusize_combo_text2.set('compact')
        self.outputsize_combo2.current(0)

        self.interval_label2 = ttk.Label(self.fIntraday, text='Interval: ')
        self.interval_combo_text2 = StringVar()
        self.interval_combo2 = ttk.Combobox(self.fIntraday, width=10, textvariable=self.interval_combo_text2, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text2.set('15min')
        self.interval_combo2.current(2)
        #3SMA
        self.interval_label3 = ttk.Label(self.fSimpleMovingAvg, text='Interval: ')
        self.interval_combo_text3 = StringVar()
        self.interval_combo3 = ttk.Combobox(self.fSimpleMovingAvg, width=10, textvariable=self.interval_combo_text3, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text3.set('daily')
        self.interval_combo3.current(5)

        self.time_period_label3 = ttk.Label(self.fSimpleMovingAvg, text='Time period(positive int:10/20): ')
        self.time_period_text3 = StringVar(value='20')
        self.time_period_entry3 = ttk.Entry(self.fSimpleMovingAvg, textvariable=self.time_period_text3, width=3)

        self.series_type_label3 = ttk.Label(self.fSimpleMovingAvg, text='Series Type: ')
        self.series_type_combo_text3 = StringVar()
        self.series_type_combo3 = ttk.Combobox(self.fSimpleMovingAvg, width=10, textvariable=self.series_type_combo_text3, 
            values=['open', 'high', 'low', 'close'], state='readonly')
        self.series_type_combo_text3.set('close')
        self.series_type_combo3.current(3)

        #4VWAP
        self.interval_label4 = ttk.Label(self.fVolumeweightedavgprice, text='Interval: ')
        self.interval_combo_text4 = StringVar()
        self.interval_combo4 = ttk.Combobox(self.fVolumeweightedavgprice, width=10, textvariable=self.interval_combo_text4, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text4.set('5min')
        self.interval_combo4.current(1)
        #5RSI
        self.interval_label5 = ttk.Label(self.fRelativeStrengthIndex, text='Interval: ')
        self.interval_combo_text5 = StringVar()
        self.interval_combo5 = ttk.Combobox(self.fRelativeStrengthIndex, width=10, textvariable=self.interval_combo_text5, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text5.set('daily')
        self.interval_combo5.current(5)

        self.time_period_label5 = ttk.Label(self.fRelativeStrengthIndex, text='Time period(positive int:10/20): ')
        self.time_period_text5 = StringVar(value='20')
        self.time_period_entry5 = ttk.Entry(self.fRelativeStrengthIndex, textvariable=self.time_period_text5, width=3)

        self.series_type_label5 = ttk.Label(self.fRelativeStrengthIndex, text='Series Type: ')
        self.series_type_combo_text5 = StringVar()
        self.series_type_combo5 = ttk.Combobox(self.fRelativeStrengthIndex, width=10, textvariable=self.series_type_combo_text5, 
            values=['open', 'high', 'low', 'close'], state='readonly')
        self.series_type_combo_text5.set('close')
        self.series_type_combo5.current(3)
        #6ADX
        self.interval_label6 = ttk.Label(self.fAvgdirectionalmovementindex, text='Interval: ')
        self.interval_combo_text6 = StringVar()
        self.interval_combo6 = ttk.Combobox(self.fAvgdirectionalmovementindex, width=10, textvariable=self.interval_combo_text6, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text6.set('daily')
        self.interval_combo6.current(5)

        self.time_period_label6 = ttk.Label(self.fAvgdirectionalmovementindex, text='Time period(positive int:10/20): ')
        self.time_period_text6 = StringVar(value='20')
        self.time_period_entry6 = ttk.Entry(self.fAvgdirectionalmovementindex, textvariable=self.time_period_text6, width=3)
        #7STOCH
        self.interval_label7 = ttk.Label(self.fstochasticoscillator, text='Interval: ')
        self.interval_combo_text7 = StringVar()
        self.interval_combo7 = ttk.Combobox(self.fstochasticoscillator, width=10, textvariable=self.interval_combo_text7, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text7.set('daily')
        self.interval_combo7.current(5)

        self.fastkperiod_label7 = ttk.Label(self.fstochasticoscillator, text='Time period of the fastk mov avg(positive int): ')
        self.fastkperiod_text7 = StringVar(value='5')
        self.fastkperiod_entry7 = ttk.Entry(self.fstochasticoscillator, textvariable=self.fastkperiod_text7, width=3)

        self.slowkperiod_label7 = ttk.Label(self.fstochasticoscillator, text='Time period of the slowk mov avg(positive int): ')
        self.slowkperiod_text7 = StringVar(value='3')
        self.slowkperiod_entry7 = ttk.Entry(self.fstochasticoscillator, textvariable=self.slowkperiod_text7, width=3)

        self.slowdperiod_label7 = ttk.Label(self.fstochasticoscillator, text='Time period of the slowd mov avg(positive int): ')
        self.slowdperiod_text7 = StringVar(value='3')
        self.slowdperiod_entry7 = ttk.Entry(self.fstochasticoscillator, textvariable=self.slowdperiod_text7, width=3)

        self.slowkmatype_label7 = ttk.Label(self.fstochasticoscillator, text='Mov Avg type for slowk mov avg: ')
        self.slowkmatype_combo_text7 = StringVar()
        self.slowkmatype_combo7 = ttk.Combobox(self.fstochasticoscillator, width=40, textvariable=self.slowkmatype_combo_text7, 
            values=['Simple Moving Average (SMA)', 'Exponential Moving Average (EMA)',
                    'Weighted Moving Average (WMA)', 'Double Exponential Moving Average (DEMA)',
                    'Triple Exponential Moving Average (TEMA)', 'Triangular Moving Average (TRIMA)',
                    'T3 Moving Average', 'Kaufman Adaptive Moving Average (KAMA)',
                    'MESA Adaptive Moving Average (MAMA)'], state='readonly')
        self.slowkmatype_combo_text7.set('Simple Moving Average (SMA)')
        self.slowkmatype_combo7.current(0)

        self.slowdmatype_label7 = ttk.Label(self.fstochasticoscillator, text='Mov Avg type for slowd mov avg: ')
        self.slowdmatype_combo_text7 = StringVar()
        self.slowdmatype_combo7 = ttk.Combobox(self.fstochasticoscillator, width=40, textvariable=self.slowdmatype_combo_text7, 
            values=['Simple Moving Average (SMA)', 'Exponential Moving Average (EMA)',
                    'Weighted Moving Average (WMA)', 'Double Exponential Moving Average (DEMA)',
                    'Triple Exponential Moving Average (TEMA)', 'Triangular Moving Average (TRIMA)',
                    'T3 Moving Average', 'Kaufman Adaptive Moving Average (KAMA)',
                    'MESA Adaptive Moving Average (MAMA)'], state='readonly')
        self.slowdmatype_combo_text7.set('Simple Moving Average (SMA)')
        self.slowdmatype_combo7.current(0)
        #8MACD
        self.interval_label8 = ttk.Label(self.fmovingaverageconvergencedivergence, text='Interval: ')
        self.interval_combo_text8 = StringVar()
        self.interval_combo8 = ttk.Combobox(self.fmovingaverageconvergencedivergence, width=10, textvariable=self.interval_combo_text8, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text8.set('daily')
        self.interval_combo8.current(5)

        self.series_type_label8 = ttk.Label(self.fmovingaverageconvergencedivergence, text='Series Type: ')
        self.series_type_combo_text8 = StringVar()
        self.series_type_combo8 = ttk.Combobox(self.fmovingaverageconvergencedivergence, width=10, textvariable=self.series_type_combo_text8, 
            values=['open', 'high', 'low', 'close'], state='readonly')
        self.series_type_combo_text8.set('close')
        self.series_type_combo8.current(3)

        self.fastperiod_label8 = ttk.Label(self.fmovingaverageconvergencedivergence, text='Fast period(positive int): ')
        self.fastperiod_text8 = StringVar(value='12')
        self.fastperiod_entry8 = ttk.Entry(self.fmovingaverageconvergencedivergence, textvariable=self.fastperiod_text8, width=3)

        self.slowperiod_label8 = ttk.Label(self.fmovingaverageconvergencedivergence, text='Slow period(positive int): ')
        self.slowperiod_text8 = StringVar(value='26')
        self.slowperiod_entry8 = ttk.Entry(self.fmovingaverageconvergencedivergence, textvariable=self.slowperiod_text8, width=3)

        self.signalperiod_label8 = ttk.Label(self.fmovingaverageconvergencedivergence, text='Signal period(positive int): ')
        self.signalperiod_text8 = StringVar(value='9')
        self.signalperiod_entry8 = ttk.Entry(self.fmovingaverageconvergencedivergence, textvariable=self.signalperiod_text8, width=3)
        #9AROON
        self.interval_label9 = ttk.Label(self.fAroon, text='Interval: ')
        self.interval_combo_text9 = StringVar()
        self.interval_combo9 = ttk.Combobox(self.fAroon, width=10, textvariable=self.interval_combo_text9, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text9.set('daily')
        self.interval_combo9.current(5)

        self.time_period_label9 = ttk.Label(self.fAroon, text='Time period(positive int:10/20): ')
        self.time_period_text9 = StringVar(value='20')
        self.time_period_entry9 = ttk.Entry(self.fAroon, textvariable=self.time_period_text9, width=3)

        self.series_type_label9 = ttk.Label(self.fAroon, text='Series Type: ')
        self.series_type_combo_text9 = StringVar()
        self.series_type_combo9 = ttk.Combobox(self.fAroon, width=10, textvariable=self.series_type_combo_text9, 
            values=['open', 'high', 'low', 'close'], state='readonly')
        self.series_type_combo_text9.set('close')
        self.series_type_combo9.current(3)

        #10
        self.interval_label10 = ttk.Label(self.fBollingerbands, text='Interval: ')
        self.interval_combo_text10 = StringVar()
        self.interval_combo10 = ttk.Combobox(self.fBollingerbands, width=10, textvariable=self.interval_combo_text10, 
            values=['1min', '5min', '15min', '30min', '60min', 'daily', 'weekly', 'monthly'], state='readonly')
        self.interval_combo_text10.set('daily')
        self.interval_combo10.current(5)

        self.time_period_label10 = ttk.Label(self.fBollingerbands, text='Time period(positive int:10/20): ')
        self.time_period_text10 = StringVar(value='20')
        self.time_period_entry10 = ttk.Entry(self.fBollingerbands, textvariable=self.time_period_text10, width=3)

        self.series_type_label10 = ttk.Label(self.fBollingerbands, text='Series Type: ')
        self.series_type_combo_text10 = StringVar()
        self.series_type_combo10 = ttk.Combobox(self.fBollingerbands, width=10, textvariable=self.series_type_combo_text10, 
            values=['open', 'high', 'low', 'close'], state='readonly')
        self.series_type_combo_text10.set('close')
        self.series_type_combo10.current(3)

        self.nbdevup_label10 = ttk.Label(self.fBollingerbands, text='Std dev multiplier of upper band(positive int): ')
        self.nbdevup_text10 = StringVar(value='2')
        self.nbdevup_entry10 = ttk.Entry(self.fBollingerbands, textvariable=self.nbdevup_text10, width=3)

        self.nbdevdn_label10 = ttk.Label(self.fBollingerbands, text='Std dev multiplier of lower band(positive int): ')
        self.nbdevdn_text10 = StringVar(value='2')
        self.nbdevdn_entry10 = ttk.Entry(self.fBollingerbands, textvariable=self.nbdevdn_text10, width=3)

        self.matype_label10 = ttk.Label(self.fBollingerbands, text='Moving avg type: ')
        self.matype_combo_text10 = StringVar()
        self.matype_combo10 = ttk.Combobox(self.fBollingerbands, width=40, textvariable=self.matype_combo_text10, 
            values=['Simple Moving Average (SMA)', 'Exponential Moving Average (EMA)',
                    'Weighted Moving Average (WMA)', 'Double Exponential Moving Average (DEMA)',
                    'Triple Exponential Moving Average (TEMA)', 'Triangular Moving Average (TRIMA)',
                    'T3 Moving Average', 'Kaufman Adaptive Moving Average (KAMA)',
                    'MESA Adaptive Moving Average (MAMA)'], state='readonly')
        self.matype_combo_text10.set('Simple Moving Average (SMA)')
        self.matype_combo10.current(0)

        #11
        self.outputsize_label11 = ttk.Label(self.fCandlestick, text='Output size: ')
        self.outpusize_combo_text11 = StringVar()
        self.outputsize_combo11 = ttk.Combobox(self.fCandlestick, width=10, 
                    textvariable=self.outpusize_combo_text11, 
                    values=['compact', 'full'], state='readonly')
        self.outpusize_combo_text11.set('compact')
        self.outputsize_combo11.current(0)

        ####
        self.frame1.grid_configure(row=0, column=0, sticky=(N, E, S, W), padx=5, pady=5)
        self.search_symbol_label.grid_configure(row=0, column=0, sticky=(E))
        self.search_symbol_combo.grid_configure(row=0, column=1, sticky=(W))
        self.btn_search_script.grid_configure(row=0, column=2, padx=5, pady=5)
        self.btn_add_script.grid_configure(row=0, column=3, padx=5, pady=5)
        self.graph_select_label.grid_configure(row=0, column=4, sticky=(E))
        self.graph_select_combo.grid_configure(row=0, column=5, sticky=(W))
        self.from_date.grid_configure(row=0, column=6, sticky=(W))
        self.btn_show_selected_graph.grid_configure(row=0, column=7, padx=5, pady=5)
        self.btn_clear_selected_graph.grid_configure(row=0, column=8, padx=5, pady=5)
        self.btn_clear_all_graph.grid_configure(row=0, column=9, padx=5, pady=5)
        self.btn_cancel.grid_configure(row=0, column=10, padx=5, pady=5)

        self.notebook.grid_configure(row=1, column=0, sticky=(N, E, S, W), padx=5, pady=5) 
        #self.fDailyPrice.grid_configure(row=0, column=1)
        #1
        self.outputsize_label1.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.outputsize_combo1.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 
        #2
        self.outputsize_label2.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.outputsize_combo2.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 
        self.interval_label2.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.interval_combo2.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 
        #3
        self.interval_label3.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo3.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.time_period_label3.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.time_period_entry3.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.series_type_label3.grid_configure(row=0, column=4, padx=2, pady=2, sticky=(E)) 
        self.series_type_combo3.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 
        #4
        self.interval_label4.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E))
        self.interval_combo4.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 
        #5
        self.interval_label5.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo5.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.time_period_label5.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.time_period_entry5.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.series_type_label5.grid_configure(row=0, column=4, padx=2, pady=2, sticky=(E)) 
        self.series_type_combo5.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 
        #6
        self.interval_label6.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo6.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.time_period_label6.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.time_period_entry6.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 
        #7
        self.interval_label7.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo7.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.fastkperiod_label7.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.fastkperiod_entry7.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.slowkperiod_label7.grid_configure(row=0, column=4, columnspan=1, padx=2, pady=2, sticky=(E)) 
        self.slowkperiod_entry7.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 

        self.slowdperiod_label7.grid_configure(row=0, column=6, padx=2, pady=2, sticky=(E)) 
        self.slowdperiod_entry7.grid_configure(row=0, column=7, padx=2, pady=2, sticky=(W)) 

        self.slowkmatype_label7.grid_configure(row=1, column=0, padx=2, pady=2, sticky=(E)) 
        self.slowkmatype_combo7.grid_configure(row=1, column=1, columnspan=2, padx=2, pady=2, sticky=(W)) 

        self.slowdmatype_label7.grid_configure(row=1, column=4, padx=2, pady=2, sticky=(E)) 
        self.slowdmatype_combo7.grid_configure(row=1, column=5, columnspan=2, padx=2, pady=2, sticky=(W)) 

        #8
        self.interval_label8.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo8.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.series_type_label8.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.series_type_combo8.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.fastperiod_label8.grid_configure(row=0, column=4, padx=2, pady=2, sticky=(E)) 
        self.fastperiod_entry8.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 

        self.slowperiod_label8.grid_configure(row=0, column=6, padx=2, pady=2, sticky=(E)) 
        self.slowperiod_entry8.grid_configure(row=0, column=7, padx=2, pady=2, sticky=(W)) 

        self.signalperiod_label8.grid_configure(row=0, column=8, padx=2, pady=2, sticky=(E)) 
        self.signalperiod_entry8.grid_configure(row=0, column=9, padx=2, pady=2, sticky=(W)) 
        #9
        self.interval_label9.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo9.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.time_period_label9.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.time_period_entry9.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.series_type_label9.grid_configure(row=0, column=4, padx=2, pady=2, sticky=(E)) 
        self.series_type_combo9.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 

        #10
        self.interval_label10.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.interval_combo10.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        self.time_period_label10.grid_configure(row=0, column=2, padx=2, pady=2, sticky=(E)) 
        self.time_period_entry10.grid_configure(row=0, column=3, padx=2, pady=2, sticky=(W)) 

        self.series_type_label10.grid_configure(row=0, column=4, padx=2, pady=2, sticky=(E)) 
        self.series_type_combo10.grid_configure(row=0, column=5, padx=2, pady=2, sticky=(W)) 

        self.nbdevup_label10.grid_configure(row=0, column=6, padx=2, pady=2, sticky=(E)) 
        self.nbdevup_entry10.grid_configure(row=0, column=7, padx=2, pady=2, sticky=(W)) 

        self.nbdevdn_label10.grid_configure(row=0, column=8, padx=2, pady=2, sticky=(E)) 
        self.nbdevdn_entry10.grid_configure(row=0, column=9, padx=2, pady=2, sticky=(W)) 

        self.matype_label10.grid_configure(row=1, column=0, padx=2, pady=2, sticky=(E)) 
        self.matype_combo10.grid_configure(row=1, column=1, columnspan=2, padx=2, pady=2, sticky=(W)) 

        #11
        self.outputsize_label11.grid_configure(row=0, column=0, padx=2, pady=2, sticky=(E)) 
        self.outputsize_combo11.grid_configure(row=0, column=1, padx=2, pady=2, sticky=(W)) 

        ####
        self.output_canvas.get_tk_widget().grid(row=2, column=0, sticky=(N, E, S, W))
        self.toolbar_frame.grid(row=3, column=0, sticky=(N, E, W, S))
        self.toolbar.grid_configure(row=0, column=0, sticky=(N, E, S, W))


        self.frame1.grid_rowconfigure(0, weight=1)
        self.frame1.grid_columnconfigure(0, weight=1)
        self.frame1.grid_columnconfigure(1, weight=1)
        self.frame1.grid_columnconfigure(2, weight=1)
        self.frame1.grid_columnconfigure(3, weight=1)
        self.frame1.grid_columnconfigure(4, weight=1)
        self.frame1.grid_columnconfigure(5, weight=1)
        self.frame1.grid_columnconfigure(6, weight=1)
        self.frame1.grid_columnconfigure(7, weight=1)
        self.frame1.grid_columnconfigure(8, weight=1)
        self.frame1.grid_columnconfigure(9, weight=1)
        self.frame1.grid_columnconfigure(10, weight=1)
        self.frame1.grid_columnconfigure(11, weight=1)

        self.notebook.grid_rowconfigure(0, weight=1)
        self.notebook.grid_columnconfigure(0, weight=1)
        self.notebook.grid_columnconfigure(1, weight=1)
        self.notebook.grid_columnconfigure(2, weight=1)
        self.notebook.grid_columnconfigure(3, weight=1)
        self.notebook.grid_columnconfigure(4, weight=1)

        self.output_canvas.get_tk_widget().grid_columnconfigure(0, weight=1)
        self.output_canvas.get_tk_widget().grid_rowconfigure(0, weight=1)
        self.output_canvas.get_tk_widget().grid_rowconfigure(1, weight=1)
        self.output_canvas.get_tk_widget().grid_rowconfigure(2, weight=1)
        self.toolbar_frame.grid_rowconfigure(0, weight=1)
        self.toolbar_frame.grid_columnconfigure(0, weight=1)
        self.toolbar.grid_columnconfigure(0, weight=1)
        self.toolbar.grid_rowconfigure(0, weight=1)

        self.grid_columnconfigure(0, weight=1)
        self.grid_columnconfigure(1, weight=1)
        self.grid_columnconfigure(2, weight=1)
        self.grid_columnconfigure(3, weight=1)
        self.grid_columnconfigure(4, weight=1)
        self.grid_columnconfigure(5, weight=1)
        self.grid_columnconfigure(6, weight=1)
        self.grid_columnconfigure(7, weight=1)
        self.grid_rowconfigure(0, weight=1)
        self.grid_rowconfigure(1, weight=1)
        self.grid_rowconfigure(2, weight=1)
        self.grid_rowconfigure(3, weight=1)


    def InitializeWindow(self):
        if(self.bool_menucalled == False):  #check from where class was initiated 
            self.wm_title("Graphs - "+ self.script)
            self.search_symbol_combo_text.set(self.script)
            self.search_symbol_combo['values'] = (self.script)
            self.search_symbol_combo.current(0)
            self.search_symbol_combo.configure(state='disabled')
            self.btn_search_script.configure(state='disabled')
            self.btn_add_script.configure(state='disabled')
            self.graph_select_combo.current(self.graphid)
            self.graph_select_combo.event_generate('<<ComboboxSelected>>')
            self.btnShowSelectedGraph()
            if(self.graphid == 0): #will show Daily + SMA
                self.graph_select_combo.current(2)
                self.graph_select_combo.event_generate('<<ComboboxSelected>>')
                self.btnShowSelectedGraph()

    def btnSearchScript(self):
        try:
            ts = TimeSeries(self.key, output_format='pandas')

            self.searchTuple=ts.get_symbol_search(self.search_symbol_combo.get())
            
            #print(searchTuple[0].columns)
            #print(searchTuple[0].values)

            search_values_list = list()
            self.search_symbol_combo['values']=search_values_list
            for i in range(len(self.searchTuple[0].values)):
                search_values_list.append(self.searchTuple[0].values[i][0] + "--" + self.searchTuple[0].values[i][1])

            self.search_symbol_combo['values']=search_values_list
            self.search_symbol_combo.focus_force()
            self.search_symbol_combo.event_generate('<Down>')

        except Exception as e:
            msgbx.showerror("Search Symbol Error", str(e))
        
        self.btn_search_script.focus_force()

    def commandEnterKey(self, event):
        self.btnSearchScript()

    def btnAddScript(self):
        curr_selection = self.search_symbol_combo.current()
        if(curr_selection >= 0):
            self.script = self.searchTuple[0].values[curr_selection][0]
            dnewscript = dict()
            dnewscript = classAddNewModifyScript(master=self, argisadd=True, argscript=self.script, argkey=self.key).show()
            # returns dictionary - {'Symbol': 'LT.BSE', 'Price': '1000', 'Date': '2020-02-22', 'Quantity': '10', 'Commission': '1', 'Cost': '10001.0'}
            if((dnewscript != None) and (len(dnewscript['Symbol']) >0)):
                stock_name = dnewscript['Symbol']
                listnewscript = list(dnewscript.items())
                self.output_tree.get_stock_quote("", stock_name, DataFrame(), listnewscript[1][0] + '=' +listnewscript[1][1],
                                                listnewscript[2][0] + '=' + listnewscript[2][1],
                                                listnewscript[3][0] + '=' + listnewscript[3][1],
                                                listnewscript[4][0] + '=' + listnewscript[4][1],
                                                listnewscript[5][0] + '=' + listnewscript[5][1])
                #dnewscript['Price'], dnewscript['Date'], 
                #   dnewscript['Quantity'], dnewscript['Commission'], dnewscript['Cost'])
        else:
            msgbx.showerror('Get Quote', 'No script selected')
            #self.focus_force()
            return

    def OnClose(self):
        self.destroy()

    def DisableAllTabs(self):
        tabs = self.notebook.tabs()
        for i, item in enumerate(tabs):
            self.notebook.tab(item, state='disabled')

    def EnableSpecificTab(self, argTabNo):
        self.notebook.tab(argTabNo, state='normal')
        self.notebook.select(argTabNo)

    def OnGraphSelectionChanged(self, event):
        self.DisableAllTabs()
        currGraph = self.graph_select_combo.current()
        self.EnableSpecificTab(currGraph)

    def OnScriptSelectionChanged(self, event):
        self.script = self.search_symbol_combo_text.get()
        curr_selection = self.search_symbol_combo.current()
        if(curr_selection >= 0):
            self.script = self.searchTuple[0].values[curr_selection][0]

    def OnAnalysis(self):
        return
    
    def OnClearGraph(self):
        self.btnClearSelectedGraph()
        return
    
    def OnPick(self, event):
        self.event = event
        currentartist = self.event.artist
        self.graph_select_combo.current(currentartist.get_gid())
        self.graph_select_combo.update_idletasks()
        
        #self.popup_ongraph.entrycget(0, 'label')
        self.popup_ongraph.entryconfigure(0, label='Selected Graph-' + self.graph_select_combo_text.get())
        self.popup_ongraph.post(event.guiEvent.x_root, event.guiEvent.y_root)
        """currentartist = event.artist
        if( currentartist.get_gid() == 0):
            msgbx.showinfo('DailyVsSMA', 'DailyVsSMA')"""

    def DrawPoints(self, argAxes, argSourceDF, argFilterRange, argPlotFieldName, 
            argCurrGraph):
        dfEvery10 = argSourceDF.iloc[::argFilterRange, :]
        for antcounter, txt in enumerate(dfEvery10[argPlotFieldName]):
                self.showCandelAnnotation(argAxis=argAxes, argTextToShow=txt, 
                        argX=mdates.datestr2num(str(dfEvery10.index[antcounter])), 
                        argY=float(txt), argXYcoords='data', argXText=1, argYText=1, 
                        argTextcoords='offset points', 
                        argHA='right', argVA='bottom', argFontsize='xx-small', argCurrGraph=argCurrGraph)

    def LoadData(self, argCurrGraph):
        try:
            if(self.IsGraphDrawn(argCurrGraph) == True):
                msgbx.showerror('Show Graph Error', 'Graph already shown')
                return

            if(self.bool_test):
                testobj = PrepareTestData(argFolder=self.datafolderpath)
            else:
                ts = TimeSeries(self.key, output_format='pandas')
                ti = TechIndicators(self.key, output_format='pandas')
            if(argCurrGraph == 0): #load daiy
                strsize = self.outpusize_combo_text1.get()
                if(self.bool_test):
                    testobjDaily = PrepareTestData(argFolder=self.datafolderpath, argOutputSize=strsize)
                    dfdata = testobjDaily.loadDaily(self.script)
                else:
                    dfdata, dfmetadata = ts.get_daily(symbol=self.script,
                                            outputsize=strsize)

                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                #self.l1, = self.ax.plot(dfdata['4. close'], label='Close', gid=argCurrGraph)
                self.ax.plot(dfdata['4. close'], label='Close', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, '4. close', argCurrGraph)
                """dfEvery10 = dfdata.iloc[::10, :]
                for antcounter, txt in enumerate(dfEvery10['4. close']):
                        self.showCandelAnnotation(argAxis=self.ax, argTextToShow=txt, 
                                argX=mdates.datestr2num(str(dfEvery10.index[antcounter])), 
                                argY=float(txt), argXYcoords='data', argXText=1, argYText=1, 
                                argTextcoords='offset points', 
                                argHA='right', argVA='bottom', argFontsize='xx-small', argCurrGraph=argCurrGraph)"""
            elif(argCurrGraph == 1): #intra
                strsize = self.outpusize_combo_text2.get()
                if(self.bool_test):
                    testobjIntra = PrepareTestData(argFolder=self.datafolderpath, argOutputSize=strsize)                    
                    dfdata = testobjIntra.loadIntra(self.script)
                else:
                    dfdata, dfmetadata = ts.get_intraday(symbol=self.script, 
                                                interval=self.interval_combo_text2.get(), 
                                                outputsize=strsize)
                #self.l2, = self.ax.plot(dfdata['4. close'], label='Intra-day', gid=argCurrGraph)
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['4. close'], label='Intra-day', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, '4. close', argCurrGraph)
                """#return rows after every 10 counts
                dfEvery10 = dfdata.iloc[::10, :]
                for antcounter, txt in enumerate(dfEvery10['4. close']):
                        self.showCandelAnnotation(argAxis=self.ax, argTextToShow=txt, 
                                argX=mdates.datestr2num(str(dfEvery10.index[antcounter])), argY=float(dfEvery10.iloc[antcounter]['4. close']),
                                argXYcoords='data', argXText=1, argYText=1, argTextcoords='offset points', 
                                argHA='right', argVA='bottom', argFontsize='xx-small', argCurrGraph=argCurrGraph)"""
            elif(argCurrGraph == 2): #SMA
                if(self.bool_test):
                    dfdata = testobj.loadSMA(self.script)
                else:
                    dfdata, dfmetadata = ti.get_sma(symbol=self.script, 
                                            interval=self.interval_combo_text3.get(), 
                                            time_period=int(self.time_period_text3.get()), 
                                            series_type=self.series_type_combo_text3.get())
                #self.l3, = self.ax.plot(dfdata['SMA'], label='SMA', gid=argCurrGraph)
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['SMA'], label='SMA', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'SMA', argCurrGraph)
            elif(argCurrGraph == 3): #VWAP
                if(self.bool_test):
                    dfdata = testobj.loadVWMP(self.script)
                else:
                    dfdata, dfmetadata = ti.get_vwap(symbol=self.script, 
                                            interval=self.interval_combo_text4.get())
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['VWAP'], label='VWAP', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'VWAP', argCurrGraph)
            elif(argCurrGraph == 4): #RSI
                if(self.bool_test):
                    dfdata = testobj.loadRSI(self.script)
                else:
                    dfdata, dfmetadata = ti.get_rsi(symbol=self.script, 
                                            interval=self.interval_combo_text5.get(), 
                                            time_period=int(self.time_period_text5.get()),
                                            series_type=self.series_type_combo_text5.get())
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['RSI'], label='RSI', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'RSI', argCurrGraph)
            elif(argCurrGraph == 5): #ADX
                if(self.bool_test):
                    dfdata = testobj.loadADX(self.script)
                else:
                    dfdata, dfmetadata = ti.get_adx(symbol=self.script, 
                                            interval=self.interval_combo_text6.get(), 
                                            time_period=int(self.time_period_text6.get()))
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['ADX'], label='ADX', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'ADX', argCurrGraph)
            elif(argCurrGraph == 6): #STOCH
                if(self.bool_test):
                    dfdata = testobj.loadStochasticOscillator(self.script)
                else:
                    dfdata, dfmetadata = ti.get_stoch(symbol=self.script, 
                                            interval=self.interval_combo_text7.get(), 
                                            fastkperiod=int(self.fastkperiod_text7.get()), 
                                            slowkperiod=int(self.slowkperiod_text7.get()), 
                                            slowdperiod=int(self.slowdperiod_text7.get()), 
                                            slowkmatype=int(self.slowkmatype_combo_text7.get()), 
                                            slowdmatype=self.slowdmatype_combo_text7.get())
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['SlowK'], label='SlowK Mov Avg', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'SlowK', argCurrGraph)
                self.ax.plot(dfdata['SlowD'], label='SlowD Mov Avg', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'SlowD', argCurrGraph)
            elif(argCurrGraph == 7): #MACD
                if(self.bool_test):
                    dfdata = testobj.loadMACD(self.script)
                else:
                    dfdata, dfmetadata = ti.get_macd(symbol=self.script, 
                                            interval=self.interval_combo_text8.get(), 
                                            series_type=self.series_type_combo_text8.get(), 
                                            fastperiod=int(self.fastperiod_text8.get()), 
                                            slowperiod=int(self.slowperiod_text8.get()), 
                                            signalperiod=int(self.signalperiod_text8.get()))
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['MACD_Signal'], label='MACD Signal', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'MACD_Signal', argCurrGraph)
                self.ax.plot(dfdata['MACD'], label='MACD', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'MACD', argCurrGraph)
                self.ax.plot(dfdata['MACD_Hist'], label='MACD History', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'MACD_Hist', argCurrGraph)
            elif(argCurrGraph == 8): #AROON
                if(self.bool_test):
                    dfdata = testobj.loadAROON(self.script)
                else:
                    dfdata, dfmetadata = ti.get_aroon(symbol=self.script, 
                                            interval=self.interval_combo_text9.get(), 
                                            time_period=int(self.time_period_text9.get()), 
                                            series_type=self.series_type_combo_text9.get())
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['Aroon Up'], label='Aroon Up', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'Aroon Up', argCurrGraph)
                self.ax.plot(dfdata['Aroon Down'], label='Aroon Down', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'Aroon Down', argCurrGraph)
            elif(argCurrGraph == 9): #BBANDS
                if(self.bool_test):
                    dfdata = testobj.loadBBands(self.script)
                else:
                    dfdata, dfmetadata = ti.get_bbands(symbol=self.script, 
                                            interval=self.interval_combo_text10.get(), 
                                            time_period=int(self.time_period_text10.get()), 
                                            series_type=self.series_type_combo_text10.get(), 
                                            nbdevup=int(self.nbdevup_text10.get()), 
                                            nbdevdn=int(self.nbdevdn_text10.get()), 
                                            matype=self.matype_combo_text10.get())
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.ax.plot(dfdata['Real Upper Band'], label='Real Upper Band', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'Real Upper Band', argCurrGraph)
                self.ax.plot(dfdata['Real Middle Band'], label='Real Middle Band', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'Real Middle Band', argCurrGraph)
                self.ax.plot(dfdata['Real Lower Band'], label='Real Lower Band', gid=argCurrGraph, marker='D', markersize = 3, markevery=10, picker=5)
                self.DrawPoints(self.ax, dfdata, 10, 'Real Lower Band', argCurrGraph)
            elif(argCurrGraph == 10): #candlestick
                strsize = self.outpusize_combo_text11.get()
                if(self.bool_test):
                    testobjCandle = PrepareTestData(argFolder=self.datafolderpath, argOutputSize=strsize)
                    dfdata = testobjCandle.loadDaily(self.script)
                else:
                    dfdata, dfmetadata = ts.get_daily(symbol=self.script,
                                            outputsize=strsize)

                #self.l1, = self.ax.plot(dfdata['4. close'], label='Close', gid=argCurrGraph)
                dfdata = dfdata.loc[dfdata.index[:] >= self.from_date_text.get()]
                self.plotMarketDataCandleSticks(dfdata, argCurrGraph) 

            self.ax.grid(True)
            #self.ax[0].set_title(argTitle, size='xx-small')
            self.ax.legend(fontsize='small')
            self.f.tight_layout()
            self.output_canvas.draw()
            self.toolbar.update()

        except Exception as e:
            msgbx.showerror('Load Data Error', 'Error while loading data-' + str(e))
    
    def btnShowSelectedGraph(self):
        if(len(self.script) <=0 ):
            msgbx.showerror('Show Graph - Error', 'Please select a script')
            return
        currGraph = self.graph_select_combo.current()
        self.LoadData(currGraph)
        #self.PlotGraph(currGraph)

    def IsGraphDrawn(self, argCurrGraph):
        for c in self.ax.lines:
            if(c.get_gid() == argCurrGraph):
                return True
        return False

    def btnClearSelectedGraph(self):
        currGraph = self.graph_select_combo.current()
        #self.ax.lines[currGraph].remove() THIS REMOVES THE SPECIFIC GRAPH LINE
        for c in self.ax.lines:
            if(c.get_gid() == currGraph):
                c.remove()

        for c in self.ax.lines:
            if(c.get_gid() == currGraph):
                c.remove()

        ####Following block is needed for candlestick bar graphs
        isbox = False
        for b in self.ax.containers:
            for p in b.patches:
                if(p.get_gid() == currGraph):
                    isbox = True
                    p.remove()

        if(isbox == True):
            del self.ax.containers[0:]

        ####candlestick remove end
        #this will return list of annotations if any
        annotations = [child for child in self.ax.get_children() if isinstance(child, matplotlib.text.Annotation)]
        for t in annotations:
            if(t.get_gid() == currGraph):
                t.remove()
        
        annotations = [child for child in self.ax.get_children() if isinstance(child, matplotlib.text.Annotation)]

        if(len(self.ax.lines) == len(self.ax.containers) == len(annotations) == 0):
            self.btnClearAllGraph()
        else:    
            #self.ax.grid(True)
            self.ax.legend(fontsize='small')
            self.ax.relim()
            self.ax.autoscale_view()
            #for c in self.ax.lines:
            #    self.ax.draw_artist(c)
            self.f.tight_layout()
            self.output_canvas.draw()
            self.toolbar.update()

    def btnClearAllGraph(self):
        del self.ax.lines[0:]

        for c in self.ax.lines:
            c.remove()

        for c in self.ax.lines:
            c.remove()

        ####Following block is needed for candlestick bar graphs
        for b in self.ax.containers:
            for p in b.patches:
                p.remove()
        #this will return list of annotations if any
        annotations = [child for child in self.ax.get_children() if isinstance(child, matplotlib.text.Annotation)]
        for t in annotations:
            t.remove()

        del self.ax.containers[0:]

        self.f.delaxes(self.ax)

        self.ax = self.f.add_subplot(1, 1, 1)
        #self.ax.relim()
        #self.ax.autoscale_view()
        #self.ax.grid(True)
        #self.ax.legend(fontsize='small')
        self.f.tight_layout()
        self.output_canvas.draw()
        self.toolbar.update()


    def showCandelAnnotation(self, argAxis, argTextToShow, argX, argY, argXYcoords, 
                            argXText, argYText, argTextcoords, argHA, argVA, argFontsize, argCurrGraph):
        argAxis.annotate(argTextToShow, 
                xy=(argX, argY),
                xycoords=argXYcoords, 
                xytext=(argXText, argYText), 
                textcoords=argTextcoords, ha=argHA, va=argVA, fontsize=argFontsize, 
                color='red', annotation_clip=True, gid=argCurrGraph)

    def plotMarketDataCandleSticks(self, argdfScript, argCurrGraph):

        dfScript=argdfScript.rename(columns={'1. open':'Open', '2. high':'High', '3. low':'Low', '4. close':'Close', '5. volume':'Volume'})

        width=0.5
        width2=0.1
        pricesup = dfScript[dfScript.Close >= dfScript.Open]

        pricesdown = dfScript[dfScript.Close < dfScript.Open]

        rect1 = self.ax.bar(pricesup.index,pricesup.Close-pricesup.Open,width,
                            gid=argCurrGraph, bottom=pricesup.Open,color='g', label='Closed higher', picker=5)
        i = 0
        for eachrec in rect1:
            amttext=''
            if(pricesup.Low[i] != pricesup.Open[i]):
                amttext = 'O=' + '{:.2f}'.format(pricesup.Open[i])
            else:
                amttext='L=O=' + '{:.2f}'.format(pricesup.Open[i])
            
            self.showCandelAnnotation(self.ax, amttext, eachrec.xy[0], pricesup.Open[i], 
                    'data', eachrec.xy[0], pricesup.Open[i], 'data', 'right', 'bottom', 
                    'xx-small', argCurrGraph)
           
            amttext = ''
            if(pricesup.High[i] != pricesup.Close[i]):
                amttext = 'C=' + '{:.2f}'.format(pricesup.Close[i])
            else:
                amttext = 'H=C=' + '{:.2f}'.format(pricesup.Close[i])

            self.showCandelAnnotation(self.ax, amttext, eachrec.xy[0], pricesup.Close[i], 
                    'data', eachrec.xy[0], pricesup.Close[i], 'data', 'right', 'top', 
                    'xx-small', argCurrGraph)

            i += 1

        rect2 = self.ax.bar(pricesup.index,pricesup.High-pricesup.Close,width2,
                            gid=argCurrGraph, bottom=pricesup.Close,color='g', picker=5)
        i = 0
        for eachrec in rect2:
            if(pricesup.High[i] != pricesup.Close[i]):
                self.showCandelAnnotation(self.ax, 'H:' + '{:.2f}'.format(pricesup.High[i]), 
                                eachrec.xy[0], pricesup.High[i], 'data', 
                                eachrec.xy[0], pricesup.High[i], 'data', 'right', 'top', 
                                'xx-small', argCurrGraph)
            i += 1
        
        rect3 = self.ax.bar(pricesup.index,pricesup.Low-pricesup.Open,width2,
                            gid=argCurrGraph, bottom=pricesup.Open,color='g', picker=5)
        i = 0
        for eachrec in rect3:
            if(pricesup.Low[i] != pricesup.Open[i]):
                self.showCandelAnnotation(self.ax, 'L:' + '{:.2f}'.format(pricesup.Low[i]), 
                                eachrec.xy[0], pricesup.Low[i], 'data', 
                                eachrec.xy[0], pricesup.Low[i], 'data', 'right', 'bottom', 
                                'xx-small', argCurrGraph)
            i += 1

        rect4 = self.ax.bar(pricesdown.index,pricesdown.Close-pricesdown.Open,width,
                        gid=argCurrGraph, bottom=pricesdown.Open,color='black', label='Closed lower', picker=5)
        i = 0
        for eachrec in rect4:
            amttext=''
            if(pricesdown.High[i] != pricesdown.Open[i]):
                amttext = 'O=' + '{:.2f}'.format(pricesdown.Open[i])
            else:
                amttext='H=O=' + '{:.2f}'.format(pricesdown.Open[i])
            
            self.showCandelAnnotation(self.ax, amttext, eachrec.xy[0], pricesdown.Open[i], 
                        'data', eachrec.xy[0], pricesdown.Open[i], 'data', 'right', 'top', 
                        'xx-small', argCurrGraph)
            
            amttext = ''
            if(pricesdown.Low[i] != pricesdown.Close[i]):
                amttext = 'C=' + '{:.2f}'.format(pricesdown.Close[i])
            else:
                amttext = 'L=C=' + '{:.2f}'.format(pricesdown.Close[i])

            self.showCandelAnnotation(self.ax, amttext, eachrec.xy[0], pricesdown.Close[i], 
                        'data', eachrec.xy[0], pricesdown.Close[i], 'data', 'right', 'bottom', 
                        'xx-small', argCurrGraph)
            i += 1

        rect5 = self.ax.bar(pricesdown.index,pricesdown.High-pricesdown.Open,width2,
                        gid=argCurrGraph, bottom=pricesdown.Open,color='black', picker=5)
        i = 0
        for eachrec in rect5:
            if(pricesdown.High[i] != pricesdown.Open[i]):
                self.showCandelAnnotation(self.ax, 'H:' + '{:.2f}'.format(pricesdown.High[i]), 
                            eachrec.xy[0], pricesdown.High[i], 'data', eachrec.xy[0], 
                            pricesdown.High[i], 'data', 'right', 'top', 'xx-small', 
                            argCurrGraph)
            i+=1

        rect6 = self.ax.bar(pricesdown.index,pricesdown.Low-pricesdown.Close,width2, 
                    gid=argCurrGraph, bottom=pricesdown.Close,color='black', picker=5)
        i = 0
        for eachrec in rect6:
            if(pricesdown.Low[i] != pricesdown.Close[i]):
                self.showCandelAnnotation(self.ax, 'L:' + '{:.2f}'.format(pricesdown.Low[i]), 
                            eachrec.xy[0], pricesdown.Low[i], 'data', eachrec.xy[0], 
                            pricesdown.Low[i], 'data', 'right', 'top', 'xx-small', argCurrGraph)
            i+=1

if __name__ == "__main__":
    obj1 = classAllGraphs(argistestmode=True, argkey='XXXX', argscript='HDFC.BSE', argmenucalled=False, arggraphid=4)
    obj1.InitializeWindow()
    obj2 = classAllGraphs(argistestmode=True, argkey='XXXX', argmenucalled=True)
    obj2.InitializeWindow()
    input()



from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx
from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators
import pandas as pd
from pandas import DataFrame
import datetime
from datetime import date
#from matplotlib.finance import candlestick2_ohlc
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
import matplotlib.dates as mdates
from matplotlib.pyplot import Figure
from matplotlib import interactive
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk


from scripttree import ScriptTreeView

from testdata import *

class PortfolioValuation(Toplevel):
    def __init__(self, master=None, argkey=None, argscript='None', argscripttree=None, 
                argIsTest=False, argDataFolder='./scriptdata'):
        Toplevel.__init__(self, master=master)
        self.key = argkey
        self.script = argscript
        self.graphctr=1
        self.datafolderpath = argDataFolder
        
        self.wm_state(newstate='zoomed') #maximize window, this works only for Win OS
        self.wm_title("Portfolio valuation")

        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)
        #numbers for SMA
        
        self.bool_test = argIsTest

        if(self.bool_test == False):
            self.ts = TimeSeries(self.key, output_format='pandas')
            self.ti = TechIndicators(self.key, output_format='pandas')

        self.treeofscripts = argscripttree
        self.dfholdingvalues = DataFrame()
        self.dfScript = DataFrame()
        self.listscriptnames = list()
        self.ax1 = None

        self.f = Figure(figsize=(12.8,4.5), dpi=100, facecolor='w', edgecolor='k', tight_layout=True, linewidth=0.9)
        self.output_canvas=FigureCanvasTkAgg(self.f, master=self)
        self.toolbar_frame=Frame(master=self)
        self.toolbar = NavigationToolbar2Tk(self.output_canvas, self.toolbar_frame)

        self.output_canvas.get_tk_widget().grid(row=0, column=0, columnspan=17, sticky=(N, E, W, S))
        self.toolbar_frame.grid(row=1, column=0, columnspan=17, rowspan=1, sticky=(N, E, W, S))
        self.toolbar.grid(row=0, column=2, sticky=(N, W))

        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)


    def show(self):
        self.wm_deiconify()
        self.wait_window()

    def OnClose(self):
        self.destroy()

    def setCurrentValInMarketDF(self, argMarketDF, argTreeDF):
        #find the shape of self.dfholdingvalues. shape returns tuple (no of rows, no of cols)
        imax = argTreeDF.shape[0]
        i = 0

        for i in range(imax):
            if(i < imax-1): #we have still not last row
                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'PurchaseDate']=argTreeDF['PurchaseDate'][i]

                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'CumulativeQTY']=argTreeDF['CumulativeQTY'][i]

                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'PurchaseQTY']=argTreeDF['PurchaseQTY'][i]

                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'Commission']=argTreeDF['Commission'][i]

                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'CostofInvestment']=argTreeDF['CostofInvestment'][i]

                argMarketDF.loc[((argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]) & 
                                (argMarketDF.index[:] < argTreeDF['PurchaseDate'][i+1])), 'CumulativeCost']=argTreeDF['CumulativeCost'][i]

                #self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                #                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i])), 'CurrentVal'] = self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'PurchaseQTY'] * self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'Close']
            else:
                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'PurchaseDate']=argTreeDF['PurchaseDate'][i]

                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'CumulativeQTY']=argTreeDF['CumulativeQTY'][i]

                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'PurchaseQTY']=argTreeDF['PurchaseQTY'][i]

                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'Commission']=argTreeDF['Commission'][i]

                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'CostofInvestment']=argTreeDF['CostofInvestment'][i]

                argMarketDF.loc[(argMarketDF.index[:] >= argTreeDF['PurchaseDate'][i]), 'CumulativeCost']=argTreeDF['CumulativeCost'][i]

                #self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'CurrentVal'] = self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'PurchaseQTY'] * self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'Close']
            
            argMarketDF.loc[(argMarketDF.index[:] == argTreeDF['PurchaseDate'][i]), 'Status']=argTreeDF['Status'][i]
            argMarketDF.loc[(argMarketDF.index[:] == argTreeDF['PurchaseDate'][i]), 'PurchasePrice']=argTreeDF['PurchasePrice'][i]
        argMarketDF['CurrentVal'] = argMarketDF['CumulativeQTY'] * argMarketDF['Close']
        argMarketDF['ScriptName'] = argTreeDF['ScriptName'][0]

        return argMarketDF

    def changeColNameTypeofDailyTS(self, argMarketDF):
        #rename columns
        argMarketDF=argMarketDF.rename(columns={'1. open':'Open', '2. high':'High', '3. low':'Low', '4. close':'Close', '5. volume':'Volume'})
                #Add new columns
        #self.dfSMAShort=self.dfSMAShort.rename(columns={'SMA':'Short_Mean'})
        #self.dfSMALong=self.dfSMALong.rename(columns={'SMA':'Long_Mean'})

        #self.dfScript = pd.concat([self.dfScript, self.dfSMAShort, self.dfSMALong], axis=1)
        #self.dfScript.sort_index(axis=0, ascending=False, inplace=True)

        argMarketDF['ScriptName'] = ""
        argMarketDF['PurchaseDate'] = ""
        argMarketDF['PurchasePrice'] = 0.00
        argMarketDF['PurchaseQTY'] = 0.00
        argMarketDF['CumulativeQTY'] = 0.00
        argMarketDF['CostofInvestment'] = 0.00
        argMarketDF['CumulativeCost'] = 0.00
        argMarketDF['CurrentVal'] = 0.00
        argMarketDF['Commission'] = 0.00
        argMarketDF['Status'] = ""
        argMarketDF['Short_Mean']=0.00
        argMarketDF['Long_Mean']=0.00
        argMarketDF['Order'] = 0
        argMarketDF['Returns'] = 0.00
        argMarketDF['CumReturns'] = 0.00

        convert_type={'Close':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'PurchasePrice':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'PurchaseQTY':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'CumulativeQTY':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'CostofInvestment':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'CumulativeCost':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'Commission':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'CurrentVal':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'Short_Mean':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'Long_Mean':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'Returns':float}
        argMarketDF = argMarketDF.astype(convert_type)
        convert_type={'CumReturns':float}
        argMarketDF = argMarketDF.astype(convert_type)
        
        return argMarketDF

    #argDF = current data from tree for each script
    def LoadDailyPrice(self, argDF):
        try:
            if(self.bool_test):
                testobj = PrepareTestData(argFolder=self.datafolderpath, argOutputSize='full')
                tempDF = testobj.loadDaily(argDF.loc[0, 'ScriptName'])
            else:
                tempDF, meta_data = self.ts.get_daily(symbol=argDF.loc[0, 'ScriptName'], outputsize='full')
            
            tempDF.sort_index(axis=0, ascending=False, inplace=True)

            tempDF = self.changeColNameTypeofDailyTS(tempDF)
            tempDF = self.setCurrentValInMarketDF(tempDF, argDF)

            if(tempDF.empty == True):
                raise ValueError('Current data not found from market for script - ' + argDF.loc[0, 'ScriptName'])

            self.dfScript = tempDF.append(self.dfScript)

            self.plotPortfolioPerformanceAX(1, 1, 1, tempDF, argDF)

            self.f.set_tight_layout(True)
            self.output_canvas.draw()
            self.toolbar.update()

        except Exception as error:
            msgbx.showerror("Error in findScriptPerformance()", str(error))
            return

    def GetValuesForScript(self, argscript):
        allchildrows = self.treeofscripts.get_children(argscript)
        tempDF = DataFrame()
        i = 0
        for child in allchildrows:
            if(str(child).upper().find(self.treeofscripts.HOLDINGVAL) >= 0):
                row_val=self.treeofscripts.item(child, 'values')
                d = {'ScriptName': argscript, 'PurchaseDate': [row_val[1]], 'PurchasePrice':[row_val[0]], 
                    'PurchaseQTY':[row_val[2]], 'Commission':[row_val[3]], 
                    'CostofInvestment':[row_val[4]], 'CurrentValue':[row_val[5]],
                    'Status':[row_val[6]], 'CumulativeQTY':['0.00'], 'CumulativeCost':['0.00']}
                tempDF = (DataFrame(d, index=[i])).append(tempDF, ignore_index=True)
                i += 1
                
        convert_type={'PurchaseQTY':float}
        tempDF = tempDF.astype(convert_type)
        convert_type={'CumulativeQTY':float}
        tempDF = tempDF.astype(convert_type)
        convert_type={'CostofInvestment':float}
        tempDF = tempDF.astype(convert_type)
        convert_type={'CumulativeCost':float}
        tempDF = tempDF.astype(convert_type)
        convert_type={'Commission':float}
        tempDF = tempDF.astype(convert_type)
        convert_type={'PurchasePrice':float}
        tempDF = tempDF.astype(convert_type)

        tempDF.sort_values('PurchaseDate', axis=0, inplace=True, ignore_index=True)
        #tempDF.drop_duplicates(subset='PurchaseDate', keep='last', inplace=True)
        #tempDF = tempDF.loc[tempDF.index.size - 1]
        imax = tempDF.shape[0]
        sumoflastrows=0
        sumoflastcost = 0
        for i in range(imax):
            #self.dfholdingvalues['PurchaseQTY'][i]=(self.dfholdingvalues['PurchaseQTY'][i])+sumoflastrows
            #sumoflastrows=self.dfholdingvalues['PurchaseQTY'][i]
            tempDF.loc[i, 'CumulativeQTY'] = tempDF.loc[i, 'PurchaseQTY']+sumoflastrows
            tempDF.loc[i, 'CumulativeCost'] = tempDF.loc[i, 'CostofInvestment']+sumoflastcost
            sumoflastrows=tempDF.loc[i, 'CumulativeQTY']
            sumoflastcost=tempDF.loc[i, 'CumulativeCost']

        self.dfholdingvalues=self.dfholdingvalues.append(tempDF, ignore_index=True)
        self.LoadDailyPrice(tempDF)

    #method to read all data from free for all scripts
    def GetPortfolioDataFromTree(self):
        self.dfholdingvalues = DataFrame()
        self.ax1 = self.f.add_subplot(1, 1, 1, label='Portfolio performance') 

        if(self.script.lower() == 'none'):    #Need to show valuation for all
            allroot = self.treeofscripts.get_children()
            for eachroot in allroot:
                self.listscriptnames.append(str(eachroot))
                self.GetValuesForScript(eachroot)
                #self.dfholdingvalues=self.dfholdingvalues.append(tempDF, ignore_index=True)
        else:
            self.listscriptnames.append(self.script)
            self.GetValuesForScript(self.script)
            #self.dfholdingvalues=self.dfholdingvalues.append(tempDF, ignore_index=True)


    def ShowPortfolioPerformance(self):
        self.GetPortfolioDataFromTree()
    
    def plotPortfolioPerformanceAX(self, argrows, argcols, argindex, argMarketDF, argTreeDF):
        self.ax1.plot(argMarketDF.loc[argMarketDF.index[:] >= 
            #argTreeDF['PurchaseDate'][argTreeDF.shape[0]-1], 'CurrentVal'], 
            argTreeDF['PurchaseDate'][0], 'CurrentVal'], 
            label='Portfolio value-' + argTreeDF['ScriptName'][argTreeDF.shape[0]-1])
        buys= argMarketDF.loc[argMarketDF.index[:] >= 
            #argTreeDF['PurchaseDate'][argTreeDF.shape[0]-1]]
            argTreeDF['PurchaseDate'][0]]
        buys = buys[buys['Status'] != '']
        self.ax1.plot(buys.index, argMarketDF['CurrentVal'].loc[buys.index], 
            marker="*", markersize=6, label='QTY ('+argTreeDF['ScriptName'][0]+')', linestyle='None')

        for i in range(len(buys.index)):
            self.ax1.annotate('Cumu. Qty='+ str(buys['CumulativeQTY'][i]) + '\nCum Cost='+ str(buys['CumulativeCost'][i]) + '\nValue='+ str(buys['CurrentVal'][i]) +" "+buys['Status'][i], 
                        (mdates.datestr2num(buys['PurchaseDate'][i]), buys['CurrentVal'][i]),
                        xycoords='data',
                        #xytext=(mdates.datestr2num(self.getDateAfter(buys['PurchaseDate'][i])), buys['CurrentVal'][i]+2), textcoords='data', 
                        xytext=(7, 7), textcoords='offset points', 
                        arrowprops=dict(arrowstyle='-|>'),
                        horizontalalignment="left", bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='xx-small')

        self.setAxesCommonConfig(self.ax1, 'Portfolio Performance', 'Portfolio Value')

    def setAxesCommonConfig(self, argAxes, argTitle, argYlabel):
        argAxes.set_ylabel(argYlabel, fontsize = 'xx-small', color='black')
        argAxes.tick_params(direction='out', length=6, width=2, colors='black',
            grid_color='black', grid_alpha=0.5, labelsize='xx-small')
        argAxes.tick_params(axis='x', labelrotation=30)

        argAxes.grid(True)
        argAxes.set_title(argTitle, size='small')
        argAxes.legend(fontsize='xx-small')


        



#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.7 - Base version with all graphs and bug fixes. Test
#v0.6
#v0.5
#v0.4
""" Class - cBackTestSMA
will accept 
    Alpha vantage key, 
    Script Name
    start date, 
    end date, 
    timeperiod1 = (example 20 days, 
    timeperiod2 = (example 40 days)
        Note timeperiod1 < timeperiod2, 

function to buildBackTest():
    It will create Alpha Vantage object
    will get the daily adjusted time series

"""
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx
from alpha_vantage.timeseries import TimeSeries
from alpha_vantage.techindicators import TechIndicators
import pandas as pd
from pandas import DataFrame
import datetime
from datetime import date
#from matplotlib.finance import candlestick2_ohlc
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
import matplotlib.dates as mdates
from matplotlib.pyplot import Figure
from matplotlib import interactive
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk


from scripttree import ScriptTreeView

from testdata import *

class BackTestSMA(Toplevel):
    def __init__(self, master=None, argkey=None, argscript=None, argscripttree=None, 
                argavgsmall=None, argavglarge=None, arglookbackyears=1, argIsTest=False, argDataFolder='./scriptdata'):
        Toplevel.__init__(self, master=master)
        self.key = argkey
        self.script = argscript
        self.graphctr=1
        self.datafolderpath = argDataFolder

        self.wm_state(newstate='zoomed') #maximize window, this works only for Win OS
        self.wm_title("Performance graphs: " + self.script)

        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)
        #numbers for SMA
        self.avgsmall = argavgsmall
        self.avglarge = argavglarge
        self.lookbackyears=arglookbackyears
        
        self.bool_test = argIsTest

        if(self.bool_test == False):
            self.ts = TimeSeries(self.key, output_format='pandas')
            self.ti = TechIndicators(self.key, output_format='pandas')

        self.treeofscripts = argscripttree
        self.dfholdingvalues = DataFrame()
        self.dfScript = DataFrame()
        #self.dfSMAShort = DataFrame()
        #self.dfSMALong = DataFrame()

        #self.f = Figure(figsize=(12.8,9.5), dpi=100, facecolor='w', edgecolor='k', tight_layout=True, linewidth=0.9)
        self.f = Figure(figsize=(12.8,4.5), dpi=100, facecolor='w', edgecolor='k', tight_layout=True, linewidth=0.9)
        self.output_canvas=FigureCanvasTkAgg(self.f, master=self)
        self.toolbar_frame=Frame(master=self)
        self.toolbar = NavigationToolbar2Tk(self.output_canvas, self.toolbar_frame)

        self.output_canvas.get_tk_widget().grid(row=0, column=0, columnspan=17, sticky=(N, E, W, S))
        self.toolbar_frame.grid(row=1, column=0, columnspan=17, rowspan=1, sticky=(N, E, W, S))
        self.toolbar.grid(row=0, column=2, sticky=(N, W))

        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)


    def show(self):
        self.wm_deiconify()
        self.wait_window()

    def OnClose(self):
        self.destroy()

    """ method - getScriptDataFromTree
        builds a DataFrame for all the portfolio holding values extracted from Tree
        HOLDING Values are stored in following order:
        [0] = value of 'Purchase Price'
        [1] = value of 'Purchase Date'
        [2] = value of 'Purchase Qty'
        [3] = value of 'Commission Paid'
        [4] = value of 'Cost of Investment' of buying Purchase QTY of scripts on Purchase Date
        [5] = value of 'Current Value' of the QTY at today's closing price
        [6] = value of 'Status' this is either '↑' or '↓' or '↔'
    
        what is needed is the self transaction information stored in Tree under HOLDINFVAL
        Assumming there are multiple buy transactions for the current script, we need to plot
        valuation in steps.
        For example:
        on 2020-02-24 there was a buy transacation for 2 shares at cost of 1000 + X%commission
            we need to plot the graph with daily price starting from 2020-02-24
        There was one more buy transaction for the same script for 2 shares on 2020-03-10. 
        Then from this date we need to show cumulative performance graph of all shares.

        Whar we will do is we will build a DataFrame with all such rows for the specific script
        
        then merge this DataFrame with the DataFrame from Alpha
    """        
    def getScriptDataFromTree(self):
        #get HOLDINGVAL of current script from tree
        allchildrows = self.treeofscripts.get_children(self.script)
        row_val = list()
        scriptQty = 0
        self.dfholdingvalues = DataFrame()
        for child in allchildrows:
            if(str(child).upper().find(self.treeofscripts.HOLDINGVAL) >= 0):
                row_val=self.treeofscripts.item(child, 'values')
                #scriptQty += int(row_val[2])
                d = {'PurchaseDate': [row_val[1]], 'PurchasePrice':[row_val[0]], 
                    'PurchaseQTY':[row_val[2]], 'Commission':[row_val[3]], 
                    'CostofInvestment':[row_val[4]], 'CurrentValue':[row_val[5]],
                    'Status':[row_val[6]], 'CumulativeQTY':['0.00']}
                #tempDF = DataFrame.from_dict(data=d, orient='index')
                tempDF = DataFrame(d)
                #tempDF = tempDF.transpose()
                self.dfholdingvalues=self.dfholdingvalues.append(tempDF, ignore_index=True)
        #self.dfholdingvalues.set_index('PurchaseDate')
        convert_type={'PurchaseQTY':float}
        self.dfholdingvalues = self.dfholdingvalues.astype(convert_type)
        convert_type={'CumulativeQTY':float}
        self.dfholdingvalues = self.dfholdingvalues.astype(convert_type)
        convert_type={'Commission':float}
        self.dfholdingvalues = self.dfholdingvalues.astype(convert_type)
        convert_type={'PurchasePrice':float}
        self.dfholdingvalues = self.dfholdingvalues.astype(convert_type)

        self.dfholdingvalues.sort_values('PurchaseDate', axis=0, inplace=True, ignore_index=True)
        sumoflastrows=0
        imax = self.dfholdingvalues.shape[0]
        for i in range(imax):
            #self.dfholdingvalues['PurchaseQTY'][i]=(self.dfholdingvalues['PurchaseQTY'][i])+sumoflastrows
            #sumoflastrows=self.dfholdingvalues['PurchaseQTY'][i]
            self.dfholdingvalues.loc[i, 'CumulativeQTY'] = self.dfholdingvalues.loc[i, 'PurchaseQTY']+sumoflastrows
            sumoflastrows=self.dfholdingvalues.loc[i, 'CumulativeQTY']

    """ setCurrentValInMarketDF
        In this method we will need to add three columns to the Alpha returned DF
        1. Purchase QTY, which is cumulative of QTY as of date
        2. Current Value
        3. Commission Paid
        now we need to add values in these columns in the Alpha DF
            i = 0 start of self.dfholdingvalues
            imax = self.dfholdingvalues.shape()[0] //returns tuple (no of rows, no of cols)
            for each row in Alpha DF  
                first check if i < imax
                    if( alpha.date >= self.dfholdingvalues[i].purchasedate AND 
                    alpha.date < self.dfholdingvalues[i+1].purchasedate)
                        calculate the Alpha.CurretnValue = self.dfholdingvalues[i].QTY * Alpha.ClosePrice
                        set Alpha.QTY = self.dfholdingvalues[i].QTY
                        set Alpha.Commision = self.dfholdingvalues[i].commission
                    else if alpha.date < self.dfholdingvalues[i].purchasedate
                        do nothing (we must have all rows sortd by Date in Alpha)
                    else if alpha.date < self.dfholdingvalues[i+1].purchasedate
                        i += 1
                        calculate the Alpha.CurretnValue = self.dfholdingvalues[i].QTY * Alpha.ClosePrice
                        set Alpha.QTY = self.dfholdingvalues[i].QTY
                        set Alpha.Commision = self.dfholdingvalues[i].commission
                else if i == imax
                        calculate the Alpha.CurretnValue = self.dfholdingvalues[i].QTY * Alpha.ClosePrice
                        set Alpha.QTY = self.dfholdingvalues[i].QTY
                        set Alpha.Commision = self.dfholdingvalues[i].commission    """
    
    def setCurrentValInMarketDF(self):
        #find the shape of self.dfholdingvalues. shape returns tuple (no of rows, no of cols)
        imax = self.dfholdingvalues.shape[0]
        i = 0

        # we will only use data from the date of first purchase
        #self.dfScript = self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0])]
        
        #self.dfScript = self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][imax-1])]

        #for i in range(imax-1, imax):
        for i in range(imax):
            if(i < imax-1): #we have still not last row
                self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'PurchaseDate']=self.dfholdingvalues['PurchaseDate'][i]

                self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'CumulativeQTY']=self.dfholdingvalues['CumulativeQTY'][i]

                self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'PurchaseQTY']=self.dfholdingvalues['PurchaseQTY'][i]

                self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'Commission']=self.dfholdingvalues['Commission'][i]

                #self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & 
                #                (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i])), 'CurrentVal'] = self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'PurchaseQTY'] * self.dfScript.loc[((self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]) & (self.dfScript.index[:] < self.dfholdingvalues['PurchaseDate'][i+1])), 'Close']
            else:
                self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'PurchaseDate']=self.dfholdingvalues['PurchaseDate'][i]

                self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'CumulativeQTY']=self.dfholdingvalues['CumulativeQTY'][i]

                self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'PurchaseQTY']=self.dfholdingvalues['PurchaseQTY'][i]

                self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'Commission']=self.dfholdingvalues['Commission'][i]

                #self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'CurrentVal'] = self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'PurchaseQTY'] * self.dfScript.loc[(self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][i]), 'Close']
            
            self.dfScript.loc[(self.dfScript.index[:] == self.dfholdingvalues['PurchaseDate'][i]), 'Status']=self.dfholdingvalues['Status'][i]
            self.dfScript.loc[(self.dfScript.index[:] == self.dfholdingvalues['PurchaseDate'][i]), 'PurchasePrice']=self.dfholdingvalues['PurchasePrice'][i]
        self.dfScript['CurrentVal'] = self.dfScript['CumulativeQTY'] * self.dfScript['Close']

    """ changeColNameTypeofDailyTS
        d
    """   
    def changeColNameTypeofDailyTS(self):
        #rename columns
        self.dfScript=self.dfScript.rename(columns={'1. open':'Open', '2. high':'High', '3. low':'Low', '4. close':'Close', '5. volume':'Volume'})
                #Add new columns
        #self.dfSMAShort=self.dfSMAShort.rename(columns={'SMA':'Short_Mean'})
        #self.dfSMALong=self.dfSMALong.rename(columns={'SMA':'Long_Mean'})

        #self.dfScript = pd.concat([self.dfScript, self.dfSMAShort, self.dfSMALong], axis=1)
        #self.dfScript.sort_index(axis=0, ascending=False, inplace=True)

        self.dfScript['PurchaseDate'] = ""
        self.dfScript['PurchasePrice'] = 0.00
        self.dfScript['PurchaseQTY'] = 0.00
        self.dfScript['CumulativeQTY'] = 0.00
        self.dfScript['CurrentVal'] = 0.00
        self.dfScript['Commission'] = 0.00
        self.dfScript['Status'] = ""
        self.dfScript['Short_Mean']=0.00
        self.dfScript['Long_Mean']=0.00
        self.dfScript['Order'] = 0
        self.dfScript['Returns'] = 0.00
        self.dfScript['CumReturns'] = 0.00

        convert_type={'Close':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'PurchasePrice':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'PurchaseQTY':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'CumulativeQTY':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'Commission':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'CurrentVal':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'Short_Mean':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'Long_Mean':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'Returns':float}
        self.dfScript = self.dfScript.astype(convert_type)
        convert_type={'CumReturns':float}
        self.dfScript = self.dfScript.astype(convert_type)

    def addPerformance(self):
        #find the moving average as per the num of days specified by user onadjusted close
        # OLD self.dfScript['Short_Mean']=self.dfScript.rolling(self.avgsmall).mean()['Close']
        self.dfScript.loc[self.dfScript.index[:], 'Short_Mean']=self.dfScript.rolling(self.avgsmall).mean()['Close']
        # OLD self.dfScript['Long_Mean']=self.dfScript.rolling(self.avglarge).mean()['Close']
        self.dfScript.loc[self.dfScript.index[:], 'Long_Mean']=self.dfScript.rolling(self.avglarge).mean()['Close']

        #If the short moving average is now above the long moving average, 
        # then we are on a short-term upwards trend. At least that is the theory. 
        # That means, that we will buy. If the averages cross the other way around, we sell. 
        # Otherwise we do nothing.
        #This means that we make a new column which consists of a one if the short moving 
        # average is above the long moving average, otherwise it contains a zero. 
        # The shift in the last line is, because we compute on the closing prices. 
        # That means that we will then buy/sell on the next day, so we shift the buying 
        # signal one day.
        self.dfScript.loc[self.dfScript['Short_Mean'] > self.dfScript['Long_Mean'], 'Order']=1
        #self.dfScript['Order']=self.dfScript['Order'].shift(1)
        self.dfScript.loc[self.dfScript.index[:], 'Order']=self.dfScript['Order'].shift(1)

        #now we calculare relative returns for each day
        #rt=(pt−(pt−1))/(pt−1)=(pt/(pt−1))−1
        self.dfScript['Returns']=(self.dfScript['Close']/((self.dfScript['Close']).shift(1))) - 1
        #self.dfScript.loc[self.dfScript.index[:], 'Returns']=(self.dfScript['Close']/((self.dfScript['Close']).shift(1))) - 1

        #We have only returns when we trade. That means that we multiply the returns with 
        # the buying signal.
        self.dfScript['Returns']=self.dfScript['Returns']*self.dfScript['Order']
        #self.dfScript.loc[self.dfScript.index[:], 'Returns']=self.dfScript['Returns']*self.dfScript['Order']
        
        #Since we reinvest all returns, we need to take a cumulative product
        #  over the last column.
        # it=((it−1)+(it−1)*rt)=(1+rt)*(it−1),i0=1

        self.dfScript['CumReturns']=(1+self.dfScript.Returns).cumprod()
    
    
    def setAxesCommonConfig(self, argAxes, argTitle, argYlabel):
        argAxes.set_ylabel(argYlabel, fontsize = 'xx-small', color='black')
        argAxes.tick_params(direction='out', length=6, width=2, colors='black',
            grid_color='black', grid_alpha=0.5, labelsize='xx-small')
        argAxes.tick_params(axis='x', labelrotation=30)

        argAxes.grid(True)
        argAxes.set_title(argTitle, size='small')
        argAxes.legend(fontsize='xx-small')

    # argLookbackYears - is the no of years we want to go back from today
    # if today is 2020-03-23 & argLookbackYears = 1, return will be 2019-03-23
    # the expetion takes care of leap year
    def getPastDateFromDate(self, argFromDate=date.today()):
        try:
            dt = argFromDate
            dt = dt.replace(year=dt.year-self.lookbackyears)
        except ValueError:
            dt = dt.replace(year=dt.year-self.lookbackyears, day=dt.day-1)
        return str(dt)

    """getDateAfter(self, argFromDate=str(date.today()), argNoOfDays=1)
        argFromDate = date in string in YYYY-MM-DD format
        argNoOfDays = Integer that indicates number of days ahead or back
        Returns - new string in STR in yyyy-mm-dd format """
    def getDateAfter(self, argFromDate=str(date.today()), argNoOfDays=1):
        try:
            ssincedate = datetime.datetime.strptime(argFromDate, "%Y-%m-%d")
            dt = date(ssincedate.year, ssincedate.month, ssincedate.day)
            dt += datetime.timedelta(days=argNoOfDays)
        except Exception as e:
            dt = date.today()
        return str(dt)


    def plotPortfolioPerformanceAX(self, argrows, argcols, argindex):
        ax1 = self.f.add_subplot(argrows, argcols, argindex, label='Portfolio performance') 
        ax1.plot(self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1], 'CurrentVal'], 
            label='Portfolio price')
        buys= self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1]]
        buys = buys[buys['Status'] != '']
        ax1.plot(buys.index, self.dfScript['CurrentVal'].loc[buys.index], 
            marker="*", markersize=10, color='b', label='Total QTY', linestyle='None')

        for i in range(len(buys.index)):
            ax1.annotate('Total Qty='+ str(buys['CumulativeQTY'][i]) + " "+buys['Status'][i], 
                        (mdates.datestr2num(buys['PurchaseDate'][i]), buys['CurrentVal'][i]),
                        xycoords='data',
                        #xytext=(mdates.datestr2num(self.getDateAfter(buys['PurchaseDate'][i])), buys['CurrentVal'][i]+2), textcoords='data', 
                        xytext=(1, 1), textcoords='offset points', 
                        #arrowprops=dict(arrowstyle='-|>'),
                        horizontalalignment="left", bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='xx-small')
        self.setAxesCommonConfig(ax1, 'Portfolio performance - ' + self.script, "Portfolio Value")

    def showCandelAnnotation(self, argAxis, argTextToShow, argX, argY, argXYcoords, 
                            argXText, argYText, argTextcoords, argHA, argVA, argFontsize):
        argAxis.annotate(argTextToShow, 
                xy=(argX, argY),
                xycoords=argXYcoords, 
                #xytext=(argXText, argYText), textcoords=argTextcoords, 
                xytext=(1, 1), textcoords='offset points', 
                ha=argHA, va=argVA, fontsize=argFontsize, color='red', annotation_clip=True)

    def plotMarketDataCandleSticks(self, argrows, argcols, argindex):
        ax2 = self.f.add_subplot(argrows, argcols, argindex, label='Open High Low Close') 
        
        ssincedate = datetime.datetime.strptime(self.dfholdingvalues['PurchaseDate'][0], "%Y-%m-%d")
        ssincedate = date(ssincedate.year, ssincedate.month, ssincedate.day)
        
        syearpastfirst = self.getPastDateFromDate(argFromDate=ssincedate)
        width=0.5
        width2=0.1
        #'1. open':'Open', '2. high':'High', '3. low':'Low', '4. close':'Close', '5. volume':'Volume
        #pricesup = self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst]
        pricesup = self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1]]
        pricesup = pricesup[pricesup.Close >= pricesup.Open]

        #pricesdown = self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst]
        pricesdown = self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1]]
        pricesdown = pricesdown[pricesdown.Close < pricesdown.Open]

        #Axes.bar(self, x, height, width=0.8, bottom=None, *, align='center', data=None, **kwargs)[source]
        rect1 = ax2.bar(pricesup.index,pricesup.Close-pricesup.Open,width,bottom=pricesup.Open,color='g', label='Closed higher')
        i = 0
        for eachrec in rect1:
            amttext=''
            if(pricesup.Low[i] != pricesup.Open[i]):
                amttext = 'O=' + '{:.2f}'.format(pricesup.Open[i])
            else:
                amttext='L=O=' + '{:.2f}'.format(pricesup.Open[i])
            
            self.showCandelAnnotation(ax2, amttext, eachrec.xy[0], pricesup.Open[i], 'data', 
                            eachrec.xy[0], pricesup.Open[i], 'data', 'right', 'bottom', 'xx-small')
            """ax2.annotate(amttext, 
                    xy=(eachrec.xy[0], pricesup.Open[i]),
                    xycoords='data', 
                    xytext=(eachrec.xy[0], pricesup.Open[i]), 
                    textcoords='data', ha='right', va='bottom', fontsize='xx-small', annotation_clip=True)"""
            
            amttext = ''
            if(pricesup.High[i] != pricesup.Close[i]):
                amttext = 'C=' + '{:.2f}'.format(pricesup.Close[i])
            else:
                amttext = 'H=C=' + '{:.2f}'.format(pricesup.Close[i])

            self.showCandelAnnotation(ax2, amttext, eachrec.xy[0], pricesup.Close[i], 'data', 
                            eachrec.xy[0], pricesup.Close[i], 'data', 'right', 'top', 'xx-small')

            """ax2.annotate(amttext, 
                    xy=(eachrec.xy[0], pricesup.Close[i]),
                    xycoords='data', 
                    xytext=(eachrec.xy[0], pricesup.Close[i]), 
                    textcoords='data', ha='right', va='top', fontsize='xx-small', annotation_clip=True)"""
            i += 1

        rect2 = ax2.bar(pricesup.index,pricesup.High-pricesup.Close,width2,bottom=pricesup.Close,color='g')
        i = 0
        for eachrec in rect2:
            if(pricesup.High[i] != pricesup.Close[i]):
                self.showCandelAnnotation(ax2, 'H:' + '{:.2f}'.format(pricesup.High[i]), 
                                eachrec.xy[0], pricesup.High[i], 'data', 
                                eachrec.xy[0], pricesup.High[i], 'data', 'right', 'top', 'xx-small')
                """ax2.annotate('H:' + '{:.2f}'.format(pricesup.High[i]), 
                        xy=(eachrec.xy[0], pricesup.High[i]),
                        xycoords='data', 
                        xytext=(eachrec.xy[0], pricesup.High[i]), 
                        textcoords='data', ha='right', va='top', fontsize='xx-small', annotation_clip=True)"""
            i += 1
        
        rect3 = ax2.bar(pricesup.index,pricesup.Low-pricesup.Open,width2,bottom=pricesup.Open,color='g')
        i = 0
        for eachrec in rect3:
            if(pricesup.Low[i] != pricesup.Open[i]):
                self.showCandelAnnotation(ax2, 'L:' + '{:.2f}'.format(pricesup.Low[i]), 
                                eachrec.xy[0], pricesup.Low[i], 'data', 
                                eachrec.xy[0], pricesup.Low[i], 'data', 'right', 'bottom', 'xx-small')
                """ax2.annotate('L:' + '{:.2f}'.format(pricesup.Low[i]), 
                        xy=(eachrec.xy[0], pricesup.Low[i]),
                        xycoords='data', 
                        xytext=(eachrec.xy[0], pricesup.Low[i]), 
                        textcoords='data', ha='right', va='bottom', fontsize='xx-small', annotation_clip=True)"""
            i += 1

        rect4 = ax2.bar(pricesdown.index,pricesdown.Close-pricesdown.Open,width,bottom=pricesdown.Open,color='black', label='Closed lower')
        i = 0
        for eachrec in rect4:
            amttext=''
            if(pricesdown.High[i] != pricesdown.Open[i]):
                amttext = 'O=' + '{:.2f}'.format(pricesdown.Open[i])
            else:
                amttext='H=O=' + '{:.2f}'.format(pricesdown.Open[i])
            
            self.showCandelAnnotation(ax2, amttext, eachrec.xy[0], pricesdown.Open[i], 'data', 
                            eachrec.xy[0], pricesdown.Open[i], 'data', 'right', 'top', 'xx-small')
            
            amttext = ''
            if(pricesdown.Low[i] != pricesdown.Close[i]):
                amttext = 'C=' + '{:.2f}'.format(pricesdown.Close[i])
            else:
                amttext = 'L=C=' + '{:.2f}'.format(pricesdown.Close[i])

            self.showCandelAnnotation(ax2, amttext, eachrec.xy[0], pricesdown.Close[i], 'data', 
                            eachrec.xy[0], pricesdown.Close[i], 'data', 'right', 'bottom', 'xx-small')
            i += 1

        rect5 = ax2.bar(pricesdown.index,pricesdown.High-pricesdown.Open,width2,bottom=pricesdown.Open,color='black')
        i = 0
        for eachrec in rect5:
            if(pricesdown.High[i] != pricesdown.Open[i]):
                self.showCandelAnnotation(ax2, 'H:' + '{:.2f}'.format(pricesdown.High[i]), 
                                eachrec.xy[0], pricesdown.High[i], 'data', 
                                eachrec.xy[0], pricesdown.High[i], 'data', 'right', 'top', 'xx-small')
            i+=1

        rect6 = ax2.bar(pricesdown.index,pricesdown.Low-pricesdown.Close,width2, bottom=pricesdown.Close,color='black')
        i = 0
        for eachrec in rect6:
            if(pricesdown.Low[i] != pricesdown.Close[i]):
                self.showCandelAnnotation(ax2, 'L:' + '{:.2f}'.format(pricesdown.Low[i]), 
                                eachrec.xy[0], pricesdown.Low[i], 'data', 
                                eachrec.xy[0], pricesdown.Low[i], 'data', 'right', 'top', 'xx-small')
            i+=1

        #ax2.set_yticks(list(pricesdown.Open) + list(pricesup.Close))

        self.setAxesCommonConfig(ax2, 'Candlestick - ' + self.script, 'Prices')

    #plots market data year from the first purchase date
    def plotMarketData(self, argrows, argcols, argindex):
        ax3 = self.f.add_subplot(argrows, argcols, argindex, label='Market Data') 
        
        ssincedate = datetime.datetime.strptime(self.dfholdingvalues['PurchaseDate'][0], "%Y-%m-%d")
        ssincedate = date(ssincedate.year, ssincedate.month, ssincedate.day)
        
        syearpastfirst = self.getPastDateFromDate(argFromDate=ssincedate)

        #ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0], 'Close'], label='Daily Close from first purchase')
        ax3.plot(self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst, 'Close'], 
                label='Close - Year from first purchase')

        #buys= self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0]]
        buys= self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst]
        buys = buys[buys['Status'] != '']

        ax3.plot(buys.index, self.dfScript['Close'].loc[buys.index], 
            marker="*", markersize=10, color='b', label='QTY purchased', linestyle='None')
        
        for i in range(len(buys.index)):
            ax3.annotate('Qty='+ str(buys['PurchaseQTY'][i]) + " @ "+ str(buys['PurchasePrice'][i]), 
                        (mdates.datestr2num(buys['PurchaseDate'][i]), float(buys['Close'][i])),
                        xycoords='data', 
                        xytext=(1, 1), textcoords='offset points', 
                        #xytext=(mdates.datestr2num(self.getDateAfter(buys['PurchaseDate'][i])), float(buys['Close'][i])+2), textcoords='data', 
                        #arrowprops=dict(arrowstyle='-|>'),
                        horizontalalignment="left", bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='xx-small')

        #ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0], 'Short_Mean'], label='Short Mean')
        ax3.plot(self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst, 'Short_Mean'], color = 'r', label='Short Mean')
        #ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0], 'Long_Mean'], label='Long Mean')
        ax3.plot(self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst, 'Long_Mean'], color = 'g', label='Long Mean')

        #buys_suggested= self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0]]
        buys_suggested= self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst]
        buys_suggested = buys_suggested[buys_suggested['Order'] == 1]
        
        ax3.plot(buys_suggested.index, self.dfScript['Close'].loc[buys_suggested.index], 
                marker=6, markersize=5, color='b', label='Buy', linestyle='None')

        #sells_suggested=self.dfScript.loc[self.dfScript['Order'] == 0, 'Order']
        #sells_suggested= self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0]]
        sells_suggested= self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst]
        sells_suggested = sells_suggested[sells_suggested['Order'] == 0]

        ax3.plot(sells_suggested.index, self.dfScript['Close'].loc[sells_suggested.index], 
                marker=7, markersize=5, color='r', label='Sell', linestyle='None')

        self.setAxesCommonConfig(ax3, 'Market Data - ' + self.script, 'Price')

    def plotScriptReturns(self, argrows, argcols, argindex):
        ax4 = self.f.add_subplot(argrows, argcols, argindex, label='Market Data') 
        
        ssincedate = datetime.datetime.strptime(self.dfholdingvalues['PurchaseDate'][0], "%Y-%m-%d")
        ssincedate = date(ssincedate.year, ssincedate.month, ssincedate.day)
        
        syearpastfirst = self.getPastDateFromDate(argFromDate=ssincedate)

        ax4.plot(self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst, 'CumReturns'], 
                label='Cumulative Returns - Year from first purchase')

        ax4.plot(self.dfScript.loc[self.dfScript.index[:] >= syearpastfirst, 'Returns'], 
                label='Daily Returns - Year from first purchase')

        self.setAxesCommonConfig(ax4, 'Returns - ' + self.script, 'Returns')

    """ findScriptPerformance
        This method will execute Alpha
    """
    def findScriptPerformance(self, argShowPerformance=True, argShowCandlestick=True, 
                            argShowMarketData=True, argShowReturns=True):
        self.getScriptDataFromTree()
        if(self.dfholdingvalues.shape[0] < 1):
            msgbx.showwarning("Script Performance", "No script data found. Please add your purchased scripts before doing performance calculations")
            return

        try:
            if(self.bool_test):
                testobj = PrepareTestData(argFolder=self.datafolderpath, argOutputSize='full')
                self.dfScript = testobj.loadDaily(self.script)
                #self.dfSMAShort = testobj.loadSMA(self.script, self.avgsmall)
                #self.dfSMAShort = testobj.loadSMA(self.script, self.avglarge)
            else:
                self.dfScript, meta_data = self.ts.get_daily(symbol=self.script, outputsize='full')
                #self.dfSMAShort, meta_data = self.ti.get_sma(self.script, interval='daily', time_period=self.avgsmall, series_type='close')
                #self.dfSMALong, meta_data = self.ti.get_sma(self.script, interval='daily', time_period=self.avglarge, series_type='close')
            
            self.dfScript.sort_index(axis=0, ascending=False, inplace=True)
            #self.dfSMAShort.sort_index(axis=0, ascending=False, inplace=True)
            #self.dfSMALong.sort_index(axis=0, ascending=False, inplace=True)
        except Exception as error:
            msgbx.showerror("Error in findScriptPerformance()", str(error))
            return
        self.changeColNameTypeofDailyTS()
        self.setCurrentValInMarketDF()
        self.addPerformance()
        #self.plotPerformanceGraphTS()
        sumofgraphs = int(argShowPerformance) + int(argShowCandlestick) + int(argShowMarketData) + int(argShowReturns)
        nrows=1
        ncols=1
        nindex=1
        if(sumofgraphs == 4):
            nrows=ncols=2
            nindex=1
        elif(sumofgraphs == 3):
            nrows=1
            ncols=3
            nindex=1
        elif(sumofgraphs == 2):
            nrows=2
            ncols=1
            nindex=1
        elif(sumofgraphs == 1):
            nrows=1
            ncols=1
            nindex=1
        if(argShowPerformance):
            self.plotPortfolioPerformanceAX(nrows, ncols, nindex)
            nindex +=1
        if(argShowCandlestick):
            self.plotMarketDataCandleSticks(nrows, ncols, nindex)
            nindex +=1
        if(argShowMarketData):
            self.plotMarketData(nrows, ncols, nindex)
            nindex +=1
        if(argShowReturns):
            self.plotScriptReturns(nrows, ncols, nindex)
            nindex +=1

        self.f.set_tight_layout(True)
        self.output_canvas.draw()
        self.toolbar.update()

    def NOTUSED_plotPerformanceGraphTS(self):
        #first 3 & 1 means we want to show 3 graphs in 1 column
        #last 1 indicates the sequence number of the current graph
        ax1 = plt.subplot(221)
        ax1.set_label('Portfolio Performance') 
        
        # first plot the self portfolio performance using CurrentVal columns in dfScript
        ax1.plot(self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1], 'CurrentVal'], label='Portfolio price')
        #ax1.plot(self.dfScript['CurrentVal'], label='Portfolio price')
        
        # now we will put markers where the user has bought the scripts and show cumulative qty
        #buys=self.dfScript.loc[(self.dfScript['Status'] != ''), ['PurchaseDate', 'PurchaseQTY', 'Status']]
        #buys=self.dfScript.loc[(self.dfScript['Status'] != ''), :]
        buys= self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1]]
        buys = buys[buys['Status'] != '']

        ax1.plot(buys.index, self.dfScript['CurrentVal'].loc[buys.index], marker="*", markersize=5, color='b', label='Buy transaction', linestyle='None')
        for i in range(len(buys.index)):
            plt.annotate('Total Qty='+ str(buys['PurchaseQTY'][i]) + " "+buys['Status'][i], 
                        (mdates.datestr2num(buys['PurchaseDate'][i]), buys['CurrentVal'][i]),
                        xycoords='data',
                        xytext=(mdates.datestr2num(buys['PurchaseDate'][i]) + 1, buys['CurrentVal'][i]), 
                        textcoords='data', arrowprops=dict(arrowstyle='-|>'),
                        horizontalalignment="left", bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='xx-small')
        #ax1.set_ylabel("Portfolio Value")
        self.setAxesCommonConfig(ax1, 'Portfolio performance - ' + self.script, 'Portfolio Value')

        # now plot 2nd set of graph
        #ax2 = plt.subplot(312, sharex=ax1)
        ax2 = plt.subplot(222)
        ax2.set_label('One year performance') 

        sdateyearback = self.getPastDateFromDate(argFromDate=date.today())

        buys= self.dfScript.loc[self.dfScript.index[:] >= self.dfholdingvalues['PurchaseDate'][0]]
        buys = buys[buys['Status'] != '']

        ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][0], 'Close'], label='Daily Close Price for last year')
            #self.dfholdingvalues['PurchaseDate'][self.dfholdingvalues.shape[0]-1], 'Close'], label='Daily Close Price for last year')
        
        ax2.plot(buys.index, self.dfScript['Close'].loc[buys.index], marker="*", markersize=5, color='b', label='Buy transaction', linestyle='None')
        prevqty = 0
        for i in range(len(self.dfholdingvalues.index)):
            prevqty = self.dfholdingvalues['PurchaseQTY'][i] - prevqty
            plt.annotate('Qty='+ str(prevqty) + " @ "+self.dfholdingvalues['PurchasePrice'][i], 
                        (mdates.datestr2num(self.dfholdingvalues['PurchaseDate'][i]), float(self.dfholdingvalues['PurchasePrice'][i])),
                        xycoords='data', 
                        xytext=(mdates.datestr2num(self.dfholdingvalues['PurchaseDate'][i]) + 1, float(self.dfholdingvalues['PurchasePrice'][i])), 
                        textcoords='data', arrowprops=dict(arrowstyle='-|>'),
                        horizontalalignment="left", bbox=dict(boxstyle="round", facecolor="w", edgecolor="0.5", alpha=0.9), 
                        fontsize='small')
            

        ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][0], 'Short_Mean'], label='Short Mean')
        ax2.plot(self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][0], 'Long_Mean'], label='Long Mean')

        #buys_suggested=self.dfScript.loc[self.dfScript['Order'] == 1, 'Order']
        
        buys_suggested= self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][0]]
        buys_suggested = buys_suggested[buys_suggested['Order'] == 1]
        
        #plt.plot(buys.index, self.dfScript.columns['Adj Close'].loc[buys.index], marker=6, markersize=10, color='g', label='buy', linestyle='None')
        ax2.plot(buys_suggested.index, self.dfScript['Close'].loc[buys_suggested.index], marker=6, markersize=5, color='b', label='Suggested buy', linestyle='None')

        #sells_suggested=self.dfScript.loc[self.dfScript['Order'] == 0, 'Order']
        sells_suggested= self.dfScript.loc[self.dfScript.index[:] >= 
            self.dfholdingvalues['PurchaseDate'][0]]
        sells_suggested = buys_suggested[buys_suggested['Order'] == 0]


        #plt.plot(sells.index, self.dfScript.column['Adj Close'].loc[sells.index], marker=7, markersize=10, color='r', label='sell', linestyle='None')
        ax2.plot(sells_suggested.index, self.dfScript['Close'].loc[sells_suggested.index], marker=7, markersize=5, color='r', label='Suggested sell', linestyle='None')

        #ax2.set_ylabel('Price')
        self.setAxesCommonConfig(ax2, 'Market comparison - ' + self.script, 'Price')

        # Now plot 3rd set of graph for cum returns
        #ax3 = plt.subplot(313, sharex=ax1)
        ax3 = plt.subplot(223)  
        ax3.set_label('Cumulative Returns') 
        ax3.plot(self.dfScript['CumReturns'], label='Cumulative Returns')
        #ax3.set_ylabel('Cumulative Returns')
        self.setAxesCommonConfig(ax3, 'Cumulative returns - ' + self.script, 'Cumulative Returns')

        # Now plot 3rd set of graph for cum returns
        #ax3 = plt.subplot(313, sharex=ax1)
        ax4 = plt.subplot(224)
        ax4.set_label('Daily Returns') 
        ax4.plot(self.dfScript['Returns'], label='Daily Returns')
        #ax4.set_ylabel('Daily Returns')
        self.setAxesCommonConfig(ax4, 'Daily returns - ' + self.script, 'Daily Returns')
        plt.show()

        """ Method - getData(self): Not used
            get_daily_adjusted returns data and metadata in DF
            data format example: "Time Series (Daily)": {
            "2020-03-03": {
                "1. open": "173.8000",
                "2. high": "175.0000",
                "3. low": "162.2600",
                "4. close": "164.5100",
                "5. adjusted close": "164.5100",
                "6. volume": "71033645",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-03-02": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "172.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            }}
        """
    
    def getDataNotUsed(self):
        self.dfScript, meta_data = self.ts.get_daily_adjusted(self.script)
        records = {
            "2020-03-03": {
                "1. open": "173.8000",
                "2. high": "175.0000",
                "3. low": "162.2600",
                "4. close": "164.5100",
                "5. adjusted close": "164.5100",
                "6. volume": "71033645",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-03-02": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "172.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-03-01": {
                "1. open": "111.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "111.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-02-29": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "222.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-02-28": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "333.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },
            "2020-02-27": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "444.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-26": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "555.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-25": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "888.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-24": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "111.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-23": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "222.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-22": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "333.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-21": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "444.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-20": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "888.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-19": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "333.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-18": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "555.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-17": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "454.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-16": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "345.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-15": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "234.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-14": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "678.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-13": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "111.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-12": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "123.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-11": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "124.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-10": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "122.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-09": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "127.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-08": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "124.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-07": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "127.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-06": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "222.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-05": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "211.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-04": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "212.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-03": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "213.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-02": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "214.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            },"2020-02-01": {
                "1. open": "165.3100",
                "2. high": "172.9200",
                "3. low": "162.3100",
                "4. close": "172.7900",
                "5. adjusted close": "243.7900",
                "6. volume": "71030810",
                "7. dividend amount": "0.0000",
                "8. split coefficient": "1.0000"
            }}

        #print(records)
        #self.dfScript = pd.DataFrame(records.values(), columns=['1. open', "2. high", "3. low", "4. close", "5. adjusted close", 
        #            "6. volumn", "7. dividend amount", "8. split coefficient"])
        #self.dfScript.index = records.keys()
        #result = DataFrame()
        #select only the date period specified by user
        #self.dfScript.index = pd.datetime(self.dfScript)
        #self.dfScript = self.dfScript.loc[self.startdt:self.enddt, :]
        
        #Not sure if we want to filter the date by dates
        #self.dfScript = self.dfScript.loc[self.startdt:self.enddt, :]
        #print(self.dfScript)
        #We want only the adjusted close
        self.dfScript=self.dfScript[['5. adjusted close']]
        convert_type={'5. adjusted close':float}
        self.dfScript = self.dfScript.astype(convert_type)
        self.dfScript=self.dfScript.rename(columns={'5. adjusted close':'Adj Close'})
        #find the moving average as per the num of days specified by user onadjusted close
        self.dfScript['short_mean']=self.dfScript.rolling(int(self.avgsmall)).mean()['Adj Close']
        self.dfScript['long_mean']=self.dfScript.rolling(int(self.avglarge)).mean()['Adj Close']
        
        #Index is already named as "date"
        #self.dfScript=self.dfScript.rename_axis('Trans_Date')
        #print(self.dfScript)
        
        #If the short moving average is now above the long moving average, 
        # then we are on a short-term upwards trend. At least that is the theory. 
        # That means, that we will buy. If the averages cross the other way around, we sell. 
        # Otherwise we do nothing.
        #This means that we make a new column which consists of a one if the short moving 
        # average is above the long moving average, otherwise it contains a zero. 
        # The shift in the last line is, because we compute on the closing prices. 
        # That means that we will then buy/sell on the next day, so we shift the buying 
        # signal one day.
        self.dfScript['order'] = 0
        #self.dfScript['order'][self.dfScript.short_mean > self.dfScript.long_mean] = 1 #buy signal
        self.dfScript.loc[self.dfScript['short_mean'] > self.dfScript['long_mean'], 'order']=1
        self.dfScript['order']=self.dfScript['order'].shift(1)
        #now we calculare relative returns for each day
        #rt=(pt−(pt−1))/(pt−1)=(pt/(pt−1))−1
        self.dfScript['returns']=(self.dfScript['Adj Close']/((self.dfScript['Adj Close']).shift(1))) - 1
        #print(self.dfScript)
        #We have only returns when we trade. That means that we multiply the returns with 
        # the buying signal.
        self.dfScript['returns']=self.dfScript['returns']*self.dfScript['order']
        #print(self.dfScript)
        #Since we reinvest all returns, we need to take a cumulative product
        #  over the last column.
        # it=((it−1)+(it−1)*rt)=(1+rt)*(it−1),i0=1

        self.dfScript['cumreturns']=(1+self.dfScript.returns).cumprod()

        #print(self.dfScript)

        #self.dfScript.plot()
        #plt.show()
    
    """  plotgraphs - Not used
            s    """
    def plotgraphsNotUsed(self):
        f_temp=Figure(figsize=(15, 6), dpi=80, facecolor='w', edgecolor='k')

        #first plot the Adj Close graph
        ax1 = plt.subplot(311)

        #Adj Close price
        plt.plot(self.dfScript['Adj Close'], label='Adjusted close price')

        buys=self.dfScript.loc[self.dfScript['order'] == 1, 'order']
        #plt.plot(buys.index, self.dfScript.columns['Adj Close'].loc[buys.index], marker=6, markersize=10, color='g', label='buy', linestyle='None')
        plt.plot(buys.index, self.dfScript['Adj Close'].loc[buys.index], marker=6, markersize=10, color='b', label='buy', linestyle='None')

        sells=self.dfScript.loc[self.dfScript['order'] == 0, 'order']
        #plt.plot(sells.index, self.dfScript.column['Adj Close'].loc[sells.index], marker=7, markersize=10, color='r', label='sell', linestyle='None')
        plt.plot(sells.index, self.dfScript['Adj Close'].loc[sells.index], marker=7, markersize=10, color='r', label='sell', linestyle='None')

        plt.legend()    #(loc='upper left')
        plt.grid()


        #Average - short & long
        ax2 = plt.subplot(312, sharex=ax1)
        plt.plot(self.dfScript['short_mean'], label='Short Mean')
        plt.plot(self.dfScript['long_mean'], label='Long Mean')
        plt.legend()    #(loc='upper left')
        plt.grid()

        ax3 = plt.subplot(313, sharex=ax1)
        plt.plot(self.dfScript['cumreturns'], label='Cumulative Returns')
        
        plt.legend()    #(loc='upper left')
        plt.grid()

        plt.suptitle(self.script)
        plt.show()

if __name__ == "__main__":
    obj = BackTestSMA('XXXX', 'BSE:HDFC', str(date.today()), '2020-02-10', 5, 10)
    obj.getScriptDataFromTree()
    obj.getData()
    obj.plotgraphs()
    input()



#v1.0
#v0.9 - All research graph via menu & mouse click
#v0.8 - Candlestick graphs
#v0.7 - Base version with all graphs and bug fixes
#v0.6
#0.5
#v0.4
from tkinter import *
from tkinter import ttk
from tkinter import messagebox as msgbx
from datetime import date
from alpha_vantage.timeseries import TimeSeries
from tkcalendar import Calendar, DateEntry

class classAddNewModifyScript(Toplevel):
    def __init__(self, master=None, argisadd=True, argscript="", 
                argPurchasePrice="0.00", argPurchaseDate=date.today(),argPurchaseQty="0",argCommissionPaid="0.00",
                argCostofInvestment="0.00", argkey=None):
        Toplevel.__init__(self, master=master)
        
        self.isadd = argisadd
        self.script = argscript
        self.key = argkey
        
        self.wm_title("Add/Modify script")

        self.wm_protocol("WM_DELETE_WINDOW", self.OnClose)

        self.configure(padx=5, pady=10)

        self.iscancel = False

        self.frame1 = ttk.Frame(self, borderwidth=5, relief="sunken") #, width=200, height=100)

        self.search_symbol_label = ttk.Label(self, text='*Search Symbol: ')
        self.search_symbol_combo_text = StringVar()
        #self.search_symbol_combo = ttk.Combobox(self, textvariable=self.search_symbol_combo_text,state='normal', postcommand=self.commandSearchSymbol)
        self.search_symbol_combo = ttk.Combobox(self, width=60, textvariable=self.search_symbol_combo_text,state='normal')
        self.search_symbol_combo.bind('<Return>', self.commandEnterKey)

        # Now create purchase price entry
        self.price_label = ttk.Label(self.frame1, text='Enter your purchase price: ')
        self.price_text = StringVar(value=argPurchasePrice)
        self.price_entry = ttk.Entry(self.frame1, textvariable=self.price_text, width=10)

        # Now create purchase date entry
        self.purchasedate_label = ttk.Label(self.frame1, text='Enter date of purchase: ')
        self.purchasedate_text = StringVar(value=argPurchaseDate)
        #self.purchasedate_entry = ttk.Entry(self.frame1, text='yyyy-mm-dd', textvariable=self.purchasedate_text, width=10)
        self.purchasedate_entry = DateEntry(master=self.frame1, width=12, year=date.today().year, 
                                month=date.today().month, day=date.today().day, 
                                background='darkblue', foreground='white', borderwidth=2,
                                textvariable=self.purchasedate_text, date_pattern='y-m-d')

        # Now create quantity label and text box to allow user to enter stock quantity
        self.quantity_label = ttk.Label(self.frame1, text='Enter quantity purchased: ')
        self.quantity_text = StringVar(value=argPurchaseQty)
        self.quantity_entry = ttk.Entry(self.frame1, textvariable=self.quantity_text, width=5)

        # Now create commision label and text box to allow user to enter commision paid
        self.commision_label = ttk.Label(self.frame1, text='Enter commision paid to broker: ')
        self.commision_text = StringVar(value=argCommissionPaid)
        self.commision_entry = ttk.Entry(self.frame1, textvariable=self.commision_text, width=5)

        # Now create cost of purchase label tp show user cost = price*quantity+commision
        self.cost = argCostofInvestment
        self.cost_label = ttk.Label(self.frame1, text='Cost of purchase ((Price*Quantity) + Commision): ' + str(self.cost))

        # Now create buttons to Add 
        self.btn_calc_cost = ttk.Button(self.frame1, text="Calculate cost of purchase", command=self.btnCalculateCost)
        self.btn_search_script = ttk.Button(self, text="Search Script", command=self.btnSearchScript)

        # Now create buttons to Add 
        if(self.isadd == True):
            self.btn_add_script = ttk.Button(self, text="Add Script", command=self.btnAddScript)
        else:
            self.btn_add_script = ttk.Button(self, text="Modify Script", command=self.btnAddScript)

        # Now create buttons to Add 
        self.btn_cancel = ttk.Button(self, text="Cancel", command=self.btnCancel)

        if ((self.isadd == False) or (len(self.script) > 0)):
            self.search_symbol_combo_text.set(self.script)
            self.search_symbol_combo['values'] = (self.script)
            self.search_symbol_combo.current(0)
            self.search_symbol_combo.configure(state='disabled')
            self.btn_search_script.configure(state='disabled')

        #put widgets on grid_configure
        self.search_symbol_label.grid_configure(row=0, column=0, sticky=(E))
        self.search_symbol_combo.grid_configure(row=0, column=1, sticky=(W))
        self.btn_search_script.grid_configure(row=0, column=2, padx=5, pady=5)

        self.frame1.grid_configure(row=1, column=0, columnspan=3, sticky=(N, S, E, W), padx=5, pady=5)
        self.price_label.grid_configure(row=1, column=0, sticky=(E))
        self.price_entry.grid_configure(row=1, column=1, sticky=(W))
        self.purchasedate_label.grid_configure(row=1, column=2, sticky=(E))
        self.purchasedate_entry.grid_configure(row=1, column=3, sticky=(W))
        self.quantity_label.grid_configure(row=2, column=0, sticky=(E))
        self.quantity_entry.grid_configure(row=2, column=1, sticky=(W))
        self.commision_label.grid_configure(row=3, column=0, sticky=(E))
        self.commision_entry.grid_configure(row=3, column=1, sticky=(W))
        self.cost_label.grid_configure(row=4, column=0, sticky=(E))
        self.btn_calc_cost.grid_configure(row=4, column=2, padx=5, pady=5)

        self.btn_add_script.grid_configure(row=2, column=1, padx=5, pady=5)
        self.btn_cancel.grid_configure(row=2, column=2, padx=5, pady=5)

    def OnClose(self):
        self.destroy()

    def btnCalculateCost(self):
        try:
            if(len(self.commision_text.get()) == 0):
                self.commision_text.set('0.00')
            if( (len(self.price_text.get()) > 0) and (len(self.quantity_text.get()) > 0)):
                self.cost = (float(self.price_text.get()) * float(self.quantity_text.get())) + float(self.commision_text.get())
            else:
                self.cost = 0.00
            self.cost_label.configure(text = 'Cost of purchase ((Price*Quantity) + Commision): ' + str(self.cost))
        except Exception as e:
            msgbx.showerror("Error", e)        
            self.btn_calc_cost.focus_force()
            
    def btnAddScript(self):
        if((self.isadd == True) and (len(self.script) <=0)):
            curr_selection = self.search_symbol_combo.current()
            if(curr_selection >= 0):
                self.script = self.searchTuple[0].values[curr_selection][0]
                self.iscancel = False
                self.destroy()
            else:
                msgbx.showerror("Error", "Please select script to add!")
                self.search_symbol_combo.focus_force()
                return
        else:
            self.iscancel = False
            self.destroy()


    def btnCancel(self):
        self.iscancel = True
        self.destroy()

    def show(self):
        self.wm_deiconify()
        self.btn_add_script.focus_force()
        self.wait_window()
        #if(self.iscancel == True):
        #    return None
        if(self.iscancel == False):
            dictReturn = dict()
            dictReturn['Symbol'] = self.script
            dictReturn['Purchase Price'] = self.price_text.get()
            dictReturn['Purchase Date'] = self.purchasedate_text.get()
            dictReturn['Purchase Qty'] = self.quantity_text.get()
            dictReturn['Commission Paid'] = self.commision_text.get()
            dictReturn['Cost of Investment'] = str(self.cost)
            return dictReturn
        return None
    
    def btnSearchScript(self):
        try:
            ts = TimeSeries(self.key, output_format='pandas')

            self.searchTuple=ts.get_symbol_search(self.search_symbol_combo.get())
            
            #print(searchTuple[0].columns)
            #print(searchTuple[0].values)

            search_values_list = list()
            self.search_symbol_combo['values']=search_values_list
            for i in range(len(self.searchTuple[0].values)):
                search_values_list.append(self.searchTuple[0].values[i][0] + "--" + self.searchTuple[0].values[i][1])

            self.search_symbol_combo['values']=search_values_list
            self.search_symbol_combo.focus_force()
            self.search_symbol_combo.event_generate('<Down>')

        except Exception as e:
            msgbx.showerror("Search Symbol Error", str(e))
            self.search_symbol_combo.focus_force()

    def commandEnterKey(self, event):
        self.btnSearchScript()
